@@ -3,7 +3,6 @@
  * Offscreen windows can be used to transform parts of a widget
  * hierarchy. Note that the rotated button is fully functional.
  */
-#define _GNU_SOURCE
 #include <math.h>
 #include <gtk/gtk.h>
 
@@ -73,7 +72,8 @@ to_child (GtkRotatedBin *bin,
   double c, s;
   double w, h;
 
-  sincos (bin->angle, &s, &c);
+  s = sin (bin->angle);
+  c = cos (bin->angle);
   child_area = bin->child->allocation;
 
   w = c * child_area.width + s * child_area.height;
@@ -112,7 +112,8 @@ to_parent (GtkRotatedBin *bin,
   double c, s;
   double w, h;
 
-  sincos (bin->angle, &s, &c);
+  s = sin (bin->angle);
+  c = cos (bin->angle);
   child_area = bin->child->allocation;
 
   w = c * child_area.width + s * child_area.height;
@@ -165,7 +166,7 @@ gtk_rotated_bin_class_init (GtkRotatedBinClass *klass)
 static void
 gtk_rotated_bin_init (GtkRotatedBin *bin)
 {
-  GTK_WIDGET_UNSET_FLAGS (bin, GTK_NO_WINDOW);
+  gtk_widget_set_has_window (GTK_WIDGET (bin), TRUE);
 }
 
 GtkWidget *
@@ -183,7 +184,7 @@ pick_offscreen_child (GdkWindow     *offscreen_window,
  GtkAllocation child_area;
  double x, y;
 
- if (bin->child && GTK_WIDGET_VISIBLE (bin->child))
+ if (bin->child && gtk_widget_get_visible (bin->child))
     {
       to_child (bin, widget_x, widget_y, &x, &y);
 
@@ -225,12 +226,12 @@ gtk_rotated_bin_realize (GtkWidget *widget)
   GtkRotatedBin *bin = GTK_ROTATED_BIN (widget);
   GdkWindowAttr attributes;
   gint attributes_mask;
-  gint border_width;
+  guint border_width;
   GtkRequisition child_requisition;
 
-  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
+  gtk_widget_set_realized (widget, TRUE);
 
-  border_width = GTK_CONTAINER (widget)->border_width;
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
 
   attributes.x = widget->allocation.x + border_width;
   attributes.y = widget->allocation.y + border_width;
@@ -261,7 +262,7 @@ gtk_rotated_bin_realize (GtkWidget *widget)
   attributes.window_type = GDK_WINDOW_OFFSCREEN;
 
   child_requisition.width = child_requisition.height = 0;
-  if (bin->child && GTK_WIDGET_VISIBLE (bin->child))
+  if (bin->child && gtk_widget_get_visible (bin->child))
     {
       attributes.width = bin->child->allocation.width;
       attributes.height = bin->child->allocation.height;
@@ -330,7 +331,7 @@ gtk_rotated_bin_remove (GtkContainer *container,
   GtkRotatedBin *bin = GTK_ROTATED_BIN (container);
   gboolean was_visible;
 
-  was_visible = GTK_WIDGET_VISIBLE (widget);
+  was_visible = gtk_widget_get_visible (widget);
 
   if (bin->child == widget)
     {
@@ -338,7 +339,7 @@ gtk_rotated_bin_remove (GtkContainer *container,
 
       bin->child = NULL;
 
-      if (was_visible && GTK_WIDGET_VISIBLE (container))
+      if (was_visible && gtk_widget_get_visible (GTK_WIDGET (container)))
         gtk_widget_queue_resize (GTK_WIDGET (container));
     }
 }
@@ -377,19 +378,22 @@ gtk_rotated_bin_size_request (GtkWidget      *widget,
   GtkRequisition child_requisition;
   double s, c;
   double w, h;
+  guint border_width;
 
   child_requisition.width = 0;
   child_requisition.height = 0;
 
-  if (bin->child && GTK_WIDGET_VISIBLE (bin->child))
+  if (bin->child && gtk_widget_get_visible (bin->child))
     gtk_widget_size_request (bin->child, &child_requisition);
 
-  sincos (bin->angle, &s, &c);
+  s = sin (bin->angle);
+  c = cos (bin->angle);
   w = c * child_requisition.width + s * child_requisition.height;
   h = s * child_requisition.width + c * child_requisition.height;
 
-  requisition->width = GTK_CONTAINER (widget)->border_width * 2 + w;
-  requisition->height = GTK_CONTAINER (widget)->border_width * 2 + h;
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
+  requisition->width = border_width * 2 + w;
+  requisition->height = border_width * 2 + h;
 }
 
 static void
@@ -397,29 +401,30 @@ gtk_rotated_bin_size_allocate (GtkWidget     *widget,
                                GtkAllocation *allocation)
 {
   GtkRotatedBin *bin = GTK_ROTATED_BIN (widget);
-  gint border_width;
+  guint border_width;
   gint w, h;
   gdouble s, c;
 
   widget->allocation = *allocation;
 
-  border_width = GTK_CONTAINER (widget)->border_width;
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
 
   w = allocation->width - border_width * 2;
   h = allocation->height - border_width * 2;
 
-  if (GTK_WIDGET_REALIZED (widget))
+  if (gtk_widget_get_realized (widget))
     gdk_window_move_resize (widget->window,
                             allocation->x + border_width,
                             allocation->y + border_width,
                             w, h);
 
-  if (bin->child && GTK_WIDGET_VISIBLE (bin->child))
+  if (bin->child && gtk_widget_get_visible (bin->child))
     {
       GtkRequisition child_requisition;
       GtkAllocation child_allocation;
 
-      sincos (bin->angle, &s, &c);
+      s = sin (bin->angle);
+      c = cos (bin->angle);
 
       gtk_widget_get_child_requisition (bin->child, &child_requisition);
       child_allocation.x = 0;
@@ -433,7 +438,7 @@ gtk_rotated_bin_size_allocate (GtkWidget     *widget,
         child_allocation.width = MIN ((w - s * child_allocation.height) / c,
                                       (h - c * child_allocation.height) / s);
 
-      if (GTK_WIDGET_REALIZED (widget))
+      if (gtk_widget_get_realized (widget))
         gdk_window_move_resize (bin->offscreen_window,
                                 child_allocation.x,
                                 child_allocation.y,
@@ -463,7 +468,7 @@ gtk_rotated_bin_expose (GtkWidget      *widget,
   gdouble s, c;
   gdouble w, h;
 
-  if (GTK_WIDGET_DRAWABLE (widget))
+  if (gtk_widget_is_drawable (widget))
     {
       if (event->window == widget->window)
         {
@@ -471,7 +476,7 @@ gtk_rotated_bin_expose (GtkWidget      *widget,
           GtkAllocation child_area;
           cairo_t *cr;
 
-          if (bin->child && GTK_WIDGET_VISIBLE (bin->child))
+          if (bin->child && gtk_widget_get_visible (bin->child))
             {
               pixmap = gdk_offscreen_window_get_pixmap (bin->offscreen_window);
               child_area = bin->child->allocation;
@@ -479,7 +484,8 @@ gtk_rotated_bin_expose (GtkWidget      *widget,
               cr = gdk_cairo_create (widget->window);
 
               /* transform */
-              sincos (bin->angle, &s, &c);
+              s = sin (bin->angle);
+              c = cos (bin->angle);
               w = c * child_area.width + s * child_area.height;
               h = s * child_area.width + c * child_area.height;
 
@@ -562,7 +568,7 @@ do_offscreen_window (GtkWidget *do_widget)
       gtk_container_add (GTK_CONTAINER (bin), button);
     }
 
-  if (!GTK_WIDGET_VISIBLE (window))
+  if (!gtk_widget_get_visible (window))
     gtk_widget_show_all (window);
   else
     {
