@@ -1,11 +1,11 @@
-#include <gtk/gtkprintunixdialog.h>
-#include <gtk/gtkpagesetupunixdialog.h>
+#include "config.h"
+
+#include <gtk/gtkunixprint.h>
 #include <gdk/gdkkeysyms.h>
 #include <X11/Xatom.h>
 #include <gdkx.h>
 #include "widgets.h"
 
-
 #define SMALL_WIDTH  240
 #define SMALL_HEIGHT 75
 #define MEDIUM_WIDTH 240
@@ -17,7 +17,7 @@ static Window
 find_toplevel_window (Window xid)
 {
   Window root, parent, *children;
-  int nchildren;
+  guint nchildren;
 
   do
     {
@@ -279,13 +279,15 @@ create_combo_box_entry (void)
 {
   GtkWidget *widget;
   GtkWidget *align;
+  GtkWidget *child;
   
   gtk_rc_parse_string ("style \"combo-box-entry-style\" {\n"
 		       "  GtkComboBox::appears-as-list = 1\n"
 		       "}\n"
 		       "widget_class \"GtkComboBoxEntry\" style \"combo-box-entry-style\"\n" );
   widget = gtk_combo_box_entry_new_text ();
-  gtk_entry_set_text (GTK_ENTRY (GTK_BIN (widget)->child), "Combo Box Entry");
+  child = gtk_bin_get_child (GTK_BIN (widget));
+  gtk_entry_set_text (GTK_ENTRY (child), "Combo Box Entry");
   align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
   gtk_container_add (GTK_CONTAINER (align), widget);
 
@@ -397,10 +399,10 @@ create_icon_view (void)
   gtk_frame_set_shadow_type (GTK_FRAME (widget), GTK_SHADOW_IN);
   list_store = gtk_list_store_new (2, G_TYPE_STRING, GDK_TYPE_PIXBUF);
   gtk_list_store_append (list_store, &iter);
-  pixbuf = gdk_pixbuf_new_from_file ("gnome-gmush.png", NULL);
+  pixbuf = gdk_pixbuf_new_from_file ("folder.png", NULL);
   gtk_list_store_set (list_store, &iter, 0, "One", 1, pixbuf, -1);
   gtk_list_store_append (list_store, &iter);
-  pixbuf = gdk_pixbuf_new_from_file ("gnome-foot.png", NULL);
+  pixbuf = gdk_pixbuf_new_from_file ("gnome.png", NULL);
   gtk_list_store_set (list_store, &iter, 0, "Two", 1, pixbuf, -1);
 
   icon_view = gtk_icon_view_new();
@@ -612,10 +614,12 @@ create_colorsel (void)
   WidgetInfo *info;
   GtkWidget *widget;
   GtkColorSelection *colorsel;
+  GtkColorSelectionDialog *selection_dialog;
   GdkColor color;
 
   widget = gtk_color_selection_dialog_new ("Color Selection Dialog");
-  colorsel = GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (widget)->colorsel);
+  selection_dialog = GTK_COLOR_SELECTION_DIALOG (widget);
+  colorsel = GTK_COLOR_SELECTION (gtk_color_selection_dialog_get_color_selection (selection_dialog));
 
   color.red   = 0x7979;
   color.green = 0xdbdb;
@@ -729,6 +733,34 @@ create_toolbar (void)
   return new_widget_info ("toolbar", widget, SMALL);
 }
 
+static WidgetInfo *
+create_toolpalette (void)
+{
+  GtkWidget *widget, *group;
+  GtkToolItem *item;
+
+  widget = gtk_tool_palette_new ();
+  group = gtk_tool_item_group_new ("Tools");
+  gtk_container_add (GTK_CONTAINER (widget), group);
+  item = gtk_tool_button_new_from_stock (GTK_STOCK_ABOUT);
+  gtk_tool_item_group_insert (GTK_TOOL_ITEM_GROUP (group), item, -1);
+  item = gtk_tool_button_new_from_stock (GTK_STOCK_FILE);
+  gtk_tool_item_group_insert (GTK_TOOL_ITEM_GROUP (group), item, -1);
+  item = gtk_tool_button_new_from_stock (GTK_STOCK_CONNECT);
+  gtk_tool_item_group_insert (GTK_TOOL_ITEM_GROUP (group), item, -1);
+
+  group = gtk_tool_item_group_new ("More tools");
+  gtk_container_add (GTK_CONTAINER (widget), group);
+  item = gtk_tool_button_new_from_stock (GTK_STOCK_CUT);
+  gtk_tool_item_group_insert (GTK_TOOL_ITEM_GROUP (group), item, -1);
+  item = gtk_tool_button_new_from_stock (GTK_STOCK_EXECUTE);
+  gtk_tool_item_group_insert (GTK_TOOL_ITEM_GROUP (group), item, -1);
+  item = gtk_tool_button_new_from_stock (GTK_STOCK_CANCEL);
+  gtk_tool_item_group_insert (GTK_TOOL_ITEM_GROUP (group), item, -1);
+
+  return new_widget_info ("toolpalette", widget, MEDIUM);
+}
+
 static WidgetInfo *
 create_menubar (void)
 {
@@ -766,9 +798,37 @@ create_message_dialog (void)
 				   GTK_MESSAGE_INFO,
 				   GTK_BUTTONS_OK,
 				   NULL);
+  gtk_window_set_icon_name (GTK_WINDOW (widget), "gtk-copy");
   gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (widget),
 				 "<b>Message Dialog</b>\n\nWith secondary text");
-  return new_widget_info ("messagedialog", widget, MEDIUM);
+  return new_widget_info ("messagedialog", widget, ASIS);
+}
+
+static WidgetInfo *
+create_about_dialog (void)
+{
+  GtkWidget *widget;
+  const gchar *authors[] = {
+    "Peter Mattis",
+    "Spencer Kimball",
+    "Josh MacDonald",
+    "and many more...",
+    NULL
+  };
+
+  widget = gtk_about_dialog_new ();
+  g_object_set (widget,
+                "program-name", "GTK+ Code Demos",
+                "version", PACKAGE_VERSION,
+                "copyright", "(C) 1997-2009 The GTK+ Team",
+                "website", "http://www.gtk.org",
+                "comments", "Program to demonstrate GTK+ functions.",
+                "logo-icon-name", "gtk-about",
+                "title", "About GTK+ Code Demos",
+                "authors", authors,
+		NULL);
+  gtk_window_set_icon_name (GTK_WINDOW (widget), "gtk-about");
+  return new_widget_info ("aboutdialog", widget, ASIS);
 }
 
 static WidgetInfo *
@@ -911,6 +971,26 @@ create_image (void)
   return new_widget_info ("image", vbox, SMALL);
 }
 
+static WidgetInfo *
+create_spinner (void)
+{
+  GtkWidget *widget;
+  GtkWidget *align, *vbox;
+
+  widget = gtk_spinner_new ();
+  gtk_widget_set_size_request (widget, 24, 24);
+
+  vbox = gtk_vbox_new (FALSE, 3);
+  align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+  gtk_container_add (GTK_CONTAINER (align), widget);
+  gtk_box_pack_start (GTK_BOX (vbox), align, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox),
+		      gtk_label_new ("Spinner"),
+		      FALSE, FALSE, 0);
+
+  return new_widget_info ("spinner", vbox, SMALL);
+}
+
 static WidgetInfo *
 create_volume_button (void)
 {
@@ -919,7 +999,7 @@ create_volume_button (void)
   button = gtk_volume_button_new ();
   gtk_scale_button_set_value (GTK_SCALE_BUTTON (button), 33);
   /* Hack: get the private dock */
-  widget = GTK_SCALE_BUTTON (button)->plus_button->parent->parent->parent;
+  widget = gtk_scale_button_get_plus_button (GTK_SCALE_BUTTON (button))->parent->parent->parent;
   gtk_widget_show_all (widget);
   return new_widget_info ("volumebutton", widget, ASIS);
 }
@@ -957,6 +1037,9 @@ get_all_widgets (void)
 {
   GList *retval = NULL;
 
+  retval = g_list_prepend (retval, create_toolpalette ());
+  retval = g_list_prepend (retval, create_spinner ());
+  retval = g_list_prepend (retval, create_about_dialog ());
   retval = g_list_prepend (retval, create_accel_label ());
   retval = g_list_prepend (retval, create_button ());
   retval = g_list_prepend (retval, create_check_button ());
