@@ -29,16 +29,18 @@
  * Written by Denis Oliver Kropp <dok@convergence.de> and
  *            Sven Neumann <sven@convergence.de>
  */
-#include <config.h>
+#include "config.h"
 #include "gdk.h"
 
 #include "gdkdirectfb.h"
 #include "gdkprivate-directfb.h"
 #include "gdkcursor.h"
-#include "gdkalias.h"
 
 #include "x-cursors.xbm"
 
+#include <directfb_version.h>
+
+
 static struct {
   const guchar *bits;
   int width, height, hotx, hoty;
@@ -228,7 +230,7 @@ gdk_cursor_new_for_display (GdkDisplay *display,GdkCursorType cursor_type)
         }
       else
         {
-          __u32  *dst;
+          u32  *dst;
           int     pitch;
 
           ret = temp->Lock (temp, DSLF_WRITE, (void**)&dst, &pitch);
@@ -266,13 +268,13 @@ gdk_cursor_new_for_display (GdkDisplay *display,GdkCursorType cursor_type)
                       gint  bit = x-mx + (y-my) * p;
                       gint mbit =    x +     y  * mp;
 
-                      __u32 color = (x-mx < 0  ||  y-my < 0  ||
+                      u32 color = (x-mx < 0  ||  y-my < 0  ||
                                      x-mx >= stock_cursors[cursor_type].width  ||
                                      y-my >= stock_cursors[cursor_type].height)
                         ? 0x00FFFFFF : (src[bit/8] & (1 << bit%8) ? 0 : 0x00FFFFFF);
 
-		      __u8  a     = color ? 0xE0 : 0xFF;
-                      __u32 alpha = mask[mbit/8] & (1 << mbit%8) ? (a << 24) : 0;
+		      u8  a     = color ? 0xE0 : 0xFF;
+                      u32 alpha = mask[mbit/8] & (1 << mbit%8) ? (a << 24) : 0;
 
                       dst[x + y*pitch] = alpha | color;
                     }
@@ -444,6 +446,9 @@ gdk_cursor_new_from_name (GdkDisplay  *display,
 
  g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
  pixbuf = gdk_pixbuf_new_from_file(name, NULL);
+ /* Prevents attempts to load stock X cursors from generating error messages */
+ if (pixbuf == NULL)
+   return NULL;
  g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), NULL);
  cursor = gdk_cursor_new_from_pixbuf (display, pixbuf, 1, 1);
  g_object_unref (pixbuf);
@@ -484,8 +489,10 @@ gdk_cursor_get_display (GdkCursor *cursor)
 }
 
 guint
-gdk_display_get_default_cursor_size (GdkDisplay    *display)
+gdk_display_get_default_cursor_size (GdkDisplay *display)
 {
+  g_return_val_if_fail (GDK_IS_DISPLAY (display), 0);
+
   return 16;
 }
 
@@ -504,8 +511,12 @@ gdk_display_get_maximal_cursor_size (GdkDisplay *display,
                                      guint       *width,
                                      guint       *height)
 {
-  *width=gdk_display_get_default_cursor_size(display);
-  *height=*width;
+  g_return_if_fail (GDK_IS_DISPLAY (display));
+
+  /* Cursor sizes in DirectFB can be large (4095x4095), but we limit this to
+     128x128 for max compatibility with the x11 backend. */
+  *width = 128;
+  *height = 128;
 }
 
 /**
@@ -533,7 +544,3 @@ gdk_display_supports_cursor_color (GdkDisplay *display)
   g_return_val_if_fail (GDK_IS_DISPLAY (display), FALSE);
   return TRUE;
 }
-
-#define __GDK_CURSOR_X11_C__
-#include "gdkaliasdef.c"
-
