@@ -36,12 +36,11 @@
 #include <stdlib.h>
 #include <string.h>
 
-
 #include "gdkdirectfb.h"
 #include "gdkprivate-directfb.h"
 
 #include "gdkkeysyms.h"
-#include "gdkalias.h"
+
 
 static struct gdk_key *gdk_keys_by_name = NULL;
 
@@ -1629,28 +1628,7 @@ gdk_directfb_translate_key (DFBInputDeviceKeyIdentifier key_id,
 				break;
 
 			case DIKT_DEAD:
-				switch (key_symbol)
-				{
-					case DIKS_DEAD_ABOVEDOT:     keyval = GDK_dead_abovedot;     break;
-					case DIKS_DEAD_ABOVERING:    keyval = GDK_dead_abovering;    break;
-					case DIKS_DEAD_ACUTE:        keyval = GDK_dead_acute;        break;
-					case DIKS_DEAD_BREVE:        keyval = GDK_dead_breve;        break;
-					case DIKS_DEAD_CARON:        keyval = GDK_dead_caron;        break;
-					case DIKS_DEAD_CEDILLA:      keyval = GDK_dead_cedilla;      break;
-					case DIKS_DEAD_CIRCUMFLEX:   keyval = GDK_dead_circumflex;   break;
-					case DIKS_DEAD_DIAERESIS:    keyval = GDK_dead_diaeresis;    break;
-					case DIKS_DEAD_DOUBLEACUTE:  keyval = GDK_dead_doubleacute;  break;
-					case DIKS_DEAD_GRAVE:        keyval = GDK_dead_grave;        break;
-					case DIKS_DEAD_IOTA:         keyval = GDK_dead_iota;         break;
-					case DIKS_DEAD_MACRON:       keyval = GDK_dead_macron;       break;
-					case DIKS_DEAD_OGONEK:       keyval = GDK_dead_ogonek;       break;
-					case DIKS_DEAD_SEMIVOICED_SOUND:
-								     keyval = GDK_dead_semivoiced_sound;                        break;
-					case DIKS_DEAD_TILDE:        keyval = GDK_dead_tilde;        break;
-					case DIKS_DEAD_VOICED_SOUND: keyval = GDK_dead_voiced_sound; break;
-					default:
-								     break;
-				}
+				/* dead keys are handled directly by directfb */
 				break;
 
 			case DIKT_CUSTOM:
@@ -1779,6 +1757,24 @@ gdk_keymap_get_caps_lock_state (GdkKeymap *keymap)
   return FALSE;
 }
 
+/**
+ * gdk_keymap_get_num_lock_state:
+ * @keymap: a #GdkKeymap
+ *
+ * Returns whether the Num Lock modifer is locked.
+ *
+ * Returns: %TRUE if Num Lock is on
+ *
+ * Since: 3.0
+ */
+gboolean
+gdk_keymap_get_num_lock_state (GdkKeymap *keymap)
+{
+  /* FIXME: Can we implement this? */
+
+  return FALSE;
+}
+
 /**
  * gdk_keymap_get_entries_for_keycode:
  * @keymap: a #GdkKeymap or %NULL to use the default keymap
@@ -2019,6 +2015,18 @@ gdk_keymap_lookup_key (GdkKeymap          *keymap,
 	return 0;
 }
 
+void
+gdk_keymap_add_virtual_modifiers (GdkKeymap       *keymap,
+                                  GdkModifierType *state)
+{
+	g_warning ("gdk_keymap_add_virtual_modifiers unimplemented \n");
+}
 
-#define __GDK_KEYS_DIRECTFB_C__
-#include "gdkaliasdef.c"
+gboolean
+gdk_keymap_map_virtual_modifiers (GdkKeymap       *keymap,
+                                  GdkModifierType *state)
+{
+  g_warning ("gdk_keymap_map_virtual_modifiers unimplemented \n");
+
+  return TRUE;
+}
