@@ -28,7 +28,7 @@
 #include "gdkcursor.h"
 #include "gdkdisplay.h"
 #include "gdkinternals.h"
-#include "gdkalias.h"
+
 
 GType
 gdk_cursor_get_type (void)
@@ -98,5 +98,19 @@ gdk_cursor_new (GdkCursorType cursor_type)
   return gdk_cursor_new_for_display (gdk_display_get_default(), cursor_type);
 }
 
-#define __GDK_CURSOR_C__
-#include "gdkaliasdef.c"
+/**
+ * gdk_cursor_get_cursor_type:
+ * @cursor:  a #GdkCursor
+ *
+ * Returns the cursor type for this cursor.
+ *
+ * Return value: a #GdkCursorType
+ *
+ * Since: 2.22
+ **/
+GdkCursorType
+gdk_cursor_get_cursor_type (GdkCursor *cursor)
+{
+  g_return_val_if_fail (cursor != NULL, GDK_BLANK_CURSOR);
+  return cursor->type;
+}
