@@ -30,7 +30,7 @@
 
 #include "gdk.h"
 #include "gdkinternals.h"
-#include "gdkalias.h"
+
 
 typedef struct _GdkIOClosure GdkIOClosure;
 
@@ -473,7 +473,7 @@ gdk_event_copy (const GdkEvent *event)
     case GDK_EXPOSE:
     case GDK_DAMAGE:
       if (event->expose.region)
-	new_event->expose.region = gdk_region_copy (event->expose.region);
+	new_event->expose.region = cairo_region_copy (event->expose.region);
       break;
       
     case GDK_SETTING:
@@ -510,8 +510,8 @@ gdk_event_copy (const GdkEvent *event)
  * 
  * Frees a #GdkEvent, freeing or decrementing any resources associated with it.
  * Note that this function should only be called with events returned from
- * functions such as gdk_event_peek(), gdk_event_get(),
- * gdk_event_get_graphics_expose(), gdk_event_copy() and gdk_event_new().
+ * functions such as gdk_event_peek(), gdk_event_get(), gdk_event_copy()
+ * and gdk_event_new().
  **/
 void
 gdk_event_free (GdkEvent *event)
@@ -551,7 +551,7 @@ gdk_event_free (GdkEvent *event)
     case GDK_EXPOSE:
     case GDK_DAMAGE:
       if (event->expose.region)
-	gdk_region_destroy (event->expose.region);
+	cairo_region_destroy (event->expose.region);
       break;
       
     case GDK_MOTION_NOTIFY:
@@ -1541,6 +1541,3 @@ gdk_setting_get (const gchar *name,
 {
   return gdk_screen_get_setting (gdk_screen_get_default (), name, value);
 }
-
-#define __GDK_EVENTS_C__
-#include "gdkaliasdef.c"
