@@ -77,30 +77,17 @@ typedef enum {
   GDK_DEBUG_XIM           = 1 << 3,
   GDK_DEBUG_NOGRABS       = 1 << 4,
   GDK_DEBUG_COLORMAP	  = 1 << 5,
-  GDK_DEBUG_GDKRGB	  = 1 << 6,
-  GDK_DEBUG_GC		  = 1 << 7,
-  GDK_DEBUG_PIXMAP	  = 1 << 8,
-  GDK_DEBUG_IMAGE	  = 1 << 9,
-  GDK_DEBUG_INPUT	  = 1 <<10,
-  GDK_DEBUG_CURSOR	  = 1 <<11,
-  GDK_DEBUG_MULTIHEAD	  = 1 <<12,
-  GDK_DEBUG_XINERAMA	  = 1 <<13,
-  GDK_DEBUG_DRAW	  = 1 <<14,
-  GDK_DEBUG_EVENTLOOP     = 1 <<15
+  GDK_DEBUG_GC		  = 1 << 6,
+  GDK_DEBUG_PIXMAP	  = 1 << 7,
+  GDK_DEBUG_IMAGE	  = 1 << 8,
+  GDK_DEBUG_INPUT	  = 1 << 9,
+  GDK_DEBUG_CURSOR	  = 1 <<10,
+  GDK_DEBUG_MULTIHEAD	  = 1 <<11,
+  GDK_DEBUG_XINERAMA	  = 1 <<12,
+  GDK_DEBUG_DRAW	  = 1 <<13,
+  GDK_DEBUG_EVENTLOOP     = 1 <<14
 } GdkDebugFlag;
 
-#ifndef GDK_DISABLE_DEPRECATED
-
-typedef struct _GdkFontPrivate	       GdkFontPrivate;
-
-struct _GdkFontPrivate
-{
-  GdkFont font;
-  guint ref_count;
-};
-
-#endif /* GDK_DISABLE_DEPRECATED */
-
 extern GList            *_gdk_default_filters;
 extern GdkWindow  	*_gdk_parent_root;
 extern gint		 _gdk_error_code;
@@ -200,9 +187,10 @@ typedef void (* GdkDisplayPointerInfoForeach) (GdkDisplay           *display,
    is public for historical reasons. Don't change that part */
 typedef struct _GdkWindowPaint             GdkWindowPaint;
 
+#define GDK_WINDOW_OBJECT(object)    ((GdkWindowObject *) GDK_WINDOW (object))
+
 struct _GdkWindowObject
 {
-  /* vvvvvvv THIS PART IS PUBLIC. DON'T CHANGE vvvvvvvvvvvvvv */
   GdkDrawable parent_instance;
 
   GdkDrawable *impl; /* window-system-specific delegate object */  
@@ -221,10 +209,11 @@ struct _GdkWindowObject
 
   GdkColor bg_color;
   GdkPixmap *bg_pixmap;
+  cairo_pattern_t *background;
   
   GSList *paint_stack;
   
-  GdkRegion *update_area;
+  cairo_region_t *update_area;
   guint update_freeze_count;
   
   guint8 window_type;
@@ -251,8 +240,6 @@ struct _GdkWindowObject
 
   GdkWindowRedirect *redirect;
 
-  /* ^^^^^^^^^^ THIS PART IS PUBLIC. DON'T CHANGE ^^^^^^^^^^ */
-  
   /* The GdkWindowObject that has the impl, ref:ed if another window.
    * This ref is required to keep the wrapper of the impl window alive
    * for as long as any GdkWindow references the impl. */
@@ -260,8 +247,8 @@ struct _GdkWindowObject
   int abs_x, abs_y; /* Absolute offset in impl */
   gint width, height;
   guint32 clip_tag;
-  GdkRegion *clip_region; /* Clip region (wrt toplevel) in window coords */
-  GdkRegion *clip_region_with_children; /* Clip region in window coords */
+  cairo_region_t *clip_region; /* Clip region (wrt toplevel) in window coords */
+  cairo_region_t *clip_region_with_children; /* Clip region in window coords */
   GdkCursor *cursor;
   GHashTable *device_cursor;
   gint8 toplevel_window_type;
@@ -278,8 +265,8 @@ struct _GdkWindowObject
 
   GList *outstanding_moves;
 
-  GdkRegion *shape;
-  GdkRegion *input_shape;
+  cairo_region_t *shape;
+  cairo_region_t *input_shape;
   
   cairo_surface_t *cairo_surface;
   guint outstanding_surfaces; /* only set on impl window */
@@ -331,31 +318,6 @@ void gdk_synthesize_window_state (GdkWindow     *window,
 GdkDeviceManager * _gdk_device_manager_new (GdkDisplay *display);
 
 
-#define GDK_SCRATCH_IMAGE_WIDTH 256
-#define GDK_SCRATCH_IMAGE_HEIGHT 64
-
-GdkImage* _gdk_image_new_for_depth (GdkScreen    *screen,
-				    GdkImageType  type,
-				    GdkVisual    *visual,
-				    gint          width,
-				    gint          height,
-				    gint          depth);
-GdkImage *_gdk_image_get_scratch (GdkScreen *screen,
-				  gint	     width,
-				  gint	     height,
-				  gint	     depth,
-				  gint	    *x,
-				  gint	    *y);
-
-GdkImage *_gdk_drawable_copy_to_image (GdkDrawable  *drawable,
-				       GdkImage     *image,
-				       gint          src_x,
-				       gint          src_y,
-				       gint          dest_x,
-				       gint          dest_y,
-				       gint          width,
-				       gint          height);
-
 cairo_surface_t *_gdk_drawable_ref_cairo_surface (GdkDrawable *drawable);
 
 GdkDrawable *_gdk_drawable_get_source_drawable (GdkDrawable *drawable);
@@ -363,18 +325,6 @@ cairo_surface_t * _gdk_drawable_create_cairo_surface (GdkDrawable *drawable,
 						      int width,
 						      int height);
 
-/* GC caching */
-GdkGC *_gdk_drawable_get_scratch_gc (GdkDrawable *drawable,
-				     gboolean     graphics_exposures);
-GdkGC *_gdk_drawable_get_subwindow_scratch_gc (GdkDrawable *drawable);
-
-void _gdk_gc_update_context (GdkGC          *gc,
-			     cairo_t        *cr,
-			     const GdkColor *override_foreground,
-			     GdkBitmap      *override_stipple,
-			     gboolean        gc_changed,
-			     GdkDrawable    *target_drawable);
-
 /*************************************
  * Interfaces used by windowing code *
  *************************************/
@@ -383,17 +333,6 @@ GdkPixmap *_gdk_pixmap_new               (GdkDrawable    *drawable,
                                           gint            width,
                                           gint            height,
                                           gint            depth);
-GdkPixmap *_gdk_pixmap_create_from_data  (GdkDrawable    *drawable,
-                                          const gchar    *data,
-                                          gint            width,
-                                          gint            height,
-                                          gint            depth,
-                                          const GdkColor *fg,
-                                          const GdkColor *bg);
-GdkPixmap *_gdk_bitmap_create_from_data  (GdkDrawable    *drawable,
-                                          const gchar    *data,
-                                          gint            width,
-                                          gint            height);
 
 void       _gdk_window_impl_new          (GdkWindow      *window,
 					  GdkWindow      *real_parent,
@@ -409,51 +348,17 @@ void       _gdk_window_update_size       (GdkWindow      *window);
 gboolean   _gdk_window_update_viewable   (GdkWindow      *window);
 
 void       _gdk_window_process_updates_recurse (GdkWindow *window,
-                                                GdkRegion *expose_region);
+                                                cairo_region_t *expose_region);
 
 void       _gdk_screen_close             (GdkScreen      *screen);
 
 const char *_gdk_get_sm_client_id (void);
 
-void _gdk_gc_init (GdkGC           *gc,
-		   GdkDrawable     *drawable,
-		   GdkGCValues     *values,
-		   GdkGCValuesMask  values_mask);
-
-GdkRegion *_gdk_gc_get_clip_region (GdkGC *gc);
-GdkBitmap *_gdk_gc_get_clip_mask   (GdkGC *gc);
-gboolean   _gdk_gc_get_exposures   (GdkGC *gc);
-GdkFill    _gdk_gc_get_fill        (GdkGC *gc);
-GdkPixmap *_gdk_gc_get_tile        (GdkGC *gc);
-GdkBitmap *_gdk_gc_get_stipple     (GdkGC *gc);
-guint32    _gdk_gc_get_fg_pixel    (GdkGC *gc);
-guint32    _gdk_gc_get_bg_pixel    (GdkGC *gc);
-void      _gdk_gc_add_drawable_clip     (GdkGC     *gc,
-					 guint32    region_tag,
-					 GdkRegion *region,
-					 int        offset_x,
-					 int        offset_y);
-void      _gdk_gc_remove_drawable_clip  (GdkGC     *gc);
-void       _gdk_gc_set_clip_region_internal (GdkGC     *gc,
-					     GdkRegion *region,
-					     gboolean reset_origin);
-GdkSubwindowMode _gdk_gc_get_subwindow (GdkGC *gc);
-
-GdkDrawable *_gdk_drawable_begin_direct_draw (GdkDrawable *drawable,
-					      GdkGC *gc,
-					      gpointer *priv_data,
-					      gint *x_offset_out,
-					      gint *y_offset_out);
-void         _gdk_drawable_end_direct_draw (gpointer priv_data);
-
-
 /*****************************************
  * Interfaces provided by windowing code *
  *****************************************/
 
 /* Font/string functions implemented in module-specific code */
-gint _gdk_font_strlen (GdkFont *font, const char *str);
-void _gdk_font_destroy (GdkFont *font);
 
 void _gdk_colormap_real_destroy (GdkColormap *colormap);
 
@@ -471,9 +376,9 @@ gulong   _gdk_windowing_window_get_next_serial  (GdkDisplay *display);
 void     _gdk_windowing_window_get_offsets      (GdkWindow  *window,
 						 gint       *x_offset,
 						 gint       *y_offset);
-GdkRegion *_gdk_windowing_window_get_shape      (GdkWindow  *window);
-GdkRegion *_gdk_windowing_window_get_input_shape(GdkWindow  *window);
-GdkRegion *_gdk_windowing_get_shape_for_mask    (GdkBitmap *mask);
+cairo_region_t *_gdk_windowing_window_get_shape      (GdkWindow  *window);
+cairo_region_t *_gdk_windowing_window_get_input_shape(GdkWindow  *window);
+cairo_region_t *_gdk_windowing_get_shape_for_mask    (GdkBitmap *mask);
 void     _gdk_windowing_window_beep             (GdkWindow *window);
 
 
@@ -503,7 +408,7 @@ void _gdk_windowing_got_event                (GdkDisplay       *display,
 					      gulong            serial);
 
 void _gdk_windowing_window_process_updates_recurse (GdkWindow *window,
-                                                    GdkRegion *expose_region);
+                                                    cairo_region_t *expose_region);
 void _gdk_windowing_before_process_all_updates     (void);
 void _gdk_windowing_after_process_all_updates      (void);
 
@@ -542,7 +447,7 @@ struct _GdkPaintableIface
   
   void (* begin_paint_region)       (GdkPaintable    *paintable,
                                      GdkWindow       *window,
-                                     const GdkRegion *region);
+                                     const cairo_region_t *region);
   void (* end_paint)                (GdkPaintable    *paintable);
 };
 
@@ -552,45 +457,6 @@ GType _gdk_paintable_get_type (void) G_GNUC_CONST;
 GType _gdk_window_impl_get_type (void) G_GNUC_CONST;
 GType _gdk_pixmap_impl_get_type (void) G_GNUC_CONST;
 
-
-/**
- * _gdk_windowing_gc_set_clip_region:
- * @gc: a #GdkGC
- * @region: the new clip region
- * @reset_origin: if TRUE, reset the clip_x/y_origin values to 0
- * 
- * Do any window-system specific processing necessary
- * for a change in clip region. Since the clip origin
- * will likely change before the GC is used with the
- * new clip, frequently this function will only set a flag and
- * do the real processing later.
- *
- * When this function is called, _gdk_gc_get_clip_region
- * will already return the new region.
- **/
-void _gdk_windowing_gc_set_clip_region (GdkGC           *gc,
-					const GdkRegion *region,
-					gboolean reset_origin);
-
-/**
- * _gdk_windowing_gc_copy:
- * @dst_gc: a #GdkGC from the GDK backend
- * @src_gc: a #GdkGC from the GDK backend
- * 
- * Copies backend specific state from @src_gc to @dst_gc.
- * This is called before the generic state is copied, so
- * the old generic state is still available from @dst_gc
- **/
-void _gdk_windowing_gc_copy (GdkGC *dst_gc,
-			     GdkGC *src_gc);
-     
-/* Queries the current foreground color of a GdkGC */
-void _gdk_windowing_gc_get_foreground (GdkGC    *gc,
-				       GdkColor *color);
-/* Queries the current background color of a GdkGC */
-void _gdk_windowing_gc_get_background (GdkGC    *gc,
-				       GdkColor *color);
-
 struct GdkAppLaunchContextPrivate
 {
   GdkDisplay *display;
@@ -644,11 +510,11 @@ void _gdk_display_pointer_info_foreach (GdkDisplay                   *display,
                                         gpointer                      user_data);
 
 void _gdk_window_invalidate_for_expose (GdkWindow       *window,
-					GdkRegion       *region);
+					cairo_region_t       *region);
 
-void _gdk_windowing_set_cairo_surface_size (cairo_surface_t *surface,
-					    int width,
-					    int height);
+gboolean _gdk_windowing_set_cairo_surface_size (cairo_surface_t *surface,
+					        int width,
+					        int height);
 
 cairo_surface_t * _gdk_windowing_create_cairo_surface (GdkDrawable *drawable,
 						       int width,
@@ -661,7 +527,7 @@ GdkWindow * _gdk_window_find_descendant_at (GdkWindow *toplevel,
 					    double *found_y);
 
 void _gdk_window_add_damage (GdkWindow *toplevel,
-			     GdkRegion *damaged_region);
+			     cairo_region_t *damaged_region);
 
 GdkEvent * _gdk_make_event (GdkWindow    *window,
 			    GdkEventType  type,
@@ -689,7 +555,7 @@ void _gdk_display_set_window_under_pointer (GdkDisplay *display,
 
 void _gdk_synthesize_crossing_events_for_geometry_change (GdkWindow *changed_window);
 
-GdkRegion *_gdk_window_calculate_full_clip_region    (GdkWindow     *window,
+cairo_region_t *_gdk_window_calculate_full_clip_region    (GdkWindow     *window,
                                                       GdkWindow     *base_window,
                                                       gboolean       do_children,
                                                       gint          *base_x_offset,
@@ -700,7 +566,6 @@ GdkWindow *_gdk_window_get_input_window_for_event (GdkWindow *native_window,
 						   GdkEventType event_type,
 						   int x, int y,
 						   gulong serial);
-GdkRegion  *_gdk_region_new_from_yxbanded_rects (GdkRectangle *rects, int n_rects);
 
 /*****************************
  * offscreen window routines *
@@ -717,7 +582,6 @@ void       _gdk_offscreen_window_new                 (GdkWindow     *window,
  * Initialization and exit routines *
  ************************************/
 
-void _gdk_image_exit  (void);
 void _gdk_windowing_exit (void);
 
 G_END_DECLS
