@@ -18,13 +18,24 @@
  * Boston, MA 02111-1307, USA.
  */
 
-
 #import "GdkQuartzView.h"
 #include "gdkwindow-quartz.h"
 #include "gdkprivate-quartz.h"
+#include "gdkquartz.h"
 
 @implementation GdkQuartzView
 
+-(void)dealloc
+{
+  if (trackingRect)
+    {
+      [self removeTrackingRect:trackingRect];
+      trackingRect = 0;
+    }
+
+  [super dealloc];
+}
+
 -(void)setGdkWindow:(GdkWindow *)window
 {
   gdk_window = window;
@@ -60,8 +71,9 @@
   GdkWindowObject *private = GDK_WINDOW_OBJECT (gdk_window);
   GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (private->impl);
   const NSRect *drawn_rects;
-  int count, i;
-  GdkRegion *region;
+  NSInteger count;
+  int i;
+  cairo_region_t *region;
 
   if (GDK_WINDOW_DESTROYED (gdk_window))
     return;
@@ -75,45 +87,28 @@
   /* Clear our own bookkeeping of regions that need display */
   if (impl->needs_display_region)
     {
-      gdk_region_destroy (impl->needs_display_region);
+      cairo_region_destroy (impl->needs_display_region);
       impl->needs_display_region = NULL;
     }
 
   [self getRectsBeingDrawn:&drawn_rects count:&count];
-
-  /* Note: arbitrary limit here to not degrade performace too much. It would
-   * be better to optimize the construction of the region below, by using
-   * _gdk_region_new_from_yxbanded_rects.
-   */
-  if (count > 25)
+  region = cairo_region_create ();
+  
+  for (i = 0; i < count; i++)
     {
-      gdk_rect.x = rect.origin.x;
-      gdk_rect.y = rect.origin.y;
-      gdk_rect.width = rect.size.width;
-      gdk_rect.height = rect.size.height;
+      gdk_rect.x = drawn_rects[i].origin.x;
+      gdk_rect.y = drawn_rects[i].origin.y;
+      gdk_rect.width = drawn_rects[i].size.width;
+      gdk_rect.height = drawn_rects[i].size.height;
 
-      region = gdk_region_rectangle (&gdk_rect);
-    }
-  else
-    {
-      region = gdk_region_new ();
-
-      for (i = 0; i < count; i++)
-        {
-          gdk_rect.x = drawn_rects[i].origin.x;
-          gdk_rect.y = drawn_rects[i].origin.y;
-          gdk_rect.width = drawn_rects[i].size.width;
-          gdk_rect.height = drawn_rects[i].size.height;
-
-          gdk_region_union_with_rect (region, &gdk_rect);
-        }
+      cairo_region_union_rectangle (region, &gdk_rect);
     }
 
   impl->in_paint_rect_count++;
   _gdk_window_process_updates_recurse (gdk_window, region);
   impl->in_paint_rect_count--;
 
-  gdk_region_destroy (region);
+  cairo_region_destroy (region);
 
   if (needsInvalidateShadow)
     {
@@ -157,6 +152,15 @@
                                  owner:self
                               userData:nil
                           assumeInside:NO];
+
+  if (NSPointInRect ([[self window] convertScreenToBase:[NSEvent mouseLocation]], rect))
+    {
+      /* When a new window (and thus view) has been created, and the mouse
+       * is in the window area, we will not receive an NSMouseEntered
+       * event.  Therefore, we synthesize an enter notify event manually.
+       */
+      _gdk_quartz_events_send_enter_notify_event (gdk_window);
+    }
 }
 
 -(void)viewDidMoveToWindow
