@@ -41,34 +41,8 @@
 
 #include "config.h"
 
-#define GDK_TYPE_GC_X11              (_gdk_gc_x11_get_type ())
-#define GDK_GC_X11(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), GDK_TYPE_GC_X11, GdkGCX11))
-#define GDK_GC_X11_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), GDK_TYPE_GC_X11, GdkGCX11Class))
-#define GDK_IS_GC_X11(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), GDK_TYPE_GC_X11))
-#define GDK_IS_GC_X11_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), GDK_TYPE_GC_X11))
-#define GDK_GC_X11_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), GDK_TYPE_GC_X11, GdkGCX11Class))
-
 typedef struct _GdkCursorPrivate       GdkCursorPrivate;
 typedef struct _GdkVisualPrivate       GdkVisualPrivate;
-typedef struct _GdkGCX11      GdkGCX11;
-typedef struct _GdkGCX11Class GdkGCX11Class;
-
-struct _GdkGCX11
-{
-  GdkGC parent_instance;
-  
-  GC xgc;
-  GdkScreen *screen;
-  guint16 dirty_mask;
-  guint have_clip_region : 1;
-  guint have_clip_mask : 1;
-  guint depth : 8;
-};
-
-struct _GdkGCX11Class
-{
-  GdkGCClass parent_class;
-};
 
 struct _GdkCursorPrivate
 {
@@ -86,7 +60,6 @@ struct _GdkVisualPrivate
   GdkScreen *screen;
 };
 
-#define XID_FONT_BIT (1<<31)
 
 void _gdk_xid_table_insert (GdkDisplay *display,
 			    XID        *xid,
@@ -99,24 +72,6 @@ gint _gdk_send_xevent      (GdkDisplay *display,
 			    glong       event_mask,
 			    XEvent     *event_send);
 
-GType _gdk_gc_x11_get_type (void);
-
-gboolean _gdk_x11_have_render           (GdkDisplay *display);
-
-GdkGC *_gdk_x11_gc_new                  (GdkDrawable     *drawable,
-					 GdkGCValues     *values,
-					 GdkGCValuesMask  values_mask);
-
-GdkImage *_gdk_x11_copy_to_image       (GdkDrawable *drawable,
-					GdkImage    *image,
-					gint         src_x,
-					gint         src_y,
-					gint         dest_x,
-					gint         dest_y,
-					gint         width,
-					gint         height);
-Pixmap   _gdk_x11_image_get_shm_pixmap (GdkImage    *image);
-
 /* Routines from gdkgeometry-x11.c */
 void _gdk_window_move_resize_child (GdkWindow     *window,
                                     gint           x,
@@ -128,21 +83,20 @@ void _gdk_window_process_expose    (GdkWindow     *window,
                                     GdkRectangle  *area);
 
 gboolean _gdk_x11_window_queue_antiexpose  (GdkWindow *window,
-					    GdkRegion *area);
-void     _gdk_x11_window_queue_translation (GdkWindow *window,
-					    GdkGC     *gc,
-					    GdkRegion *area,
+					    cairo_region_t *area);
+void     _gdk_x11_window_translate         (GdkWindow *window,
+					    cairo_region_t *area,
 					    gint       dx,
 					    gint       dy);
 
 void     _gdk_selection_window_destroyed   (GdkWindow            *window);
 gboolean _gdk_selection_filter_clear_event (XSelectionClearEvent *event);
 
-GdkRegion* _xwindow_get_shape              (Display *xdisplay,
+cairo_region_t* _xwindow_get_shape              (Display *xdisplay,
                                             Window window,
                                             gint shape_type);
 
-void     _gdk_region_get_xrectangles       (const GdkRegion      *region,
+void     _gdk_region_get_xrectangles       (const cairo_region_t      *region,
                                             gint                  x_offset,
                                             gint                  y_offset,
                                             XRectangle          **rects,
@@ -162,8 +116,6 @@ void _gdk_keymap_add_virtual_modifiers_compat (GdkKeymap       *keymap,
 gboolean _gdk_keymap_key_is_modifier   (GdkKeymap       *keymap,
 					guint            keycode);
 
-GC _gdk_x11_gc_flush (GdkGC *gc);
-
 void _gdk_x11_initialize_locale (void);
 
 void _gdk_xgrab_check_unmap        (GdkWindow *window,
@@ -177,19 +129,13 @@ void _gdk_x11_precache_atoms (GdkDisplay          *display,
 			      const gchar * const *atom_names,
 			      gint                 n_atoms);
 
-void _gdk_x11_events_init_screen   (GdkScreen *screen);
-void _gdk_x11_events_uninit_screen (GdkScreen *screen);
+void _gdk_screen_x11_events_init   (GdkScreen *screen);
 
 void _gdk_events_init           (GdkDisplay *display);
 void _gdk_events_uninit         (GdkDisplay *display);
 void _gdk_windowing_window_init (GdkScreen *screen);
 void _gdk_visual_init           (GdkScreen *screen);
 void _gdk_dnd_init		(GdkDisplay *display);
-void _gdk_windowing_image_init  (GdkDisplay *display);
-void _gdk_input_init            (GdkDisplay *display);
-
-PangoRenderer *_gdk_x11_renderer_get (GdkDrawable *drawable,
-				      GdkGC       *gc);
 
 void _gdk_x11_cursor_update_theme (GdkCursor *cursor);
 void _gdk_x11_cursor_display_finalize (GdkDisplay *display);
@@ -198,6 +144,9 @@ gboolean _gdk_x11_get_xft_setting (GdkScreen   *screen,
 				   const gchar *name,
 				   GValue      *value);
 
+GdkGrabStatus _gdk_x11_convert_grab_status (gint status);
+
+
 extern GdkDrawableClass  _gdk_x11_drawable_class;
 extern gboolean	         _gdk_use_xshm;
 extern const int         _gdk_nenvent_masks;
