@@ -17,14 +17,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include "gtkaccelmap.h"
 
 #include "gtkmarshalers.h"
 #include "gtkwindow.h"  /* in lack of GtkAcceleratable */
-#include "gtkintl.h" 
-#include "gtkalias.h"
+#include "gtkintl.h"
 
 #include <glib/gstdio.h>
 
@@ -37,7 +36,6 @@
 #ifdef G_OS_WIN32
 #include <io.h>
 #endif
-#include <errno.h>
 
 
 /* --- structures --- */
@@ -150,6 +148,10 @@ _gtk_accel_path_is_valid (const gchar *accel_path)
  * e.g. "File/Save As", "Image/View/Zoom" or "Edit/Select All".
  * So a full valid accelerator path may look like:
  * "&lt;Gimp-Toolbox&gt;/File/Dialogs/Tool Options...".
+ * 
+ * Note that @accel_path string will be stored in a #GQuark. Therefore, if you
+ * pass a static string, you can save some memory by interning it first with 
+ * g_intern_static_string().
  */
 void
 gtk_accel_map_add_entry (const gchar    *accel_path,
@@ -195,9 +197,9 @@ gtk_accel_map_add_entry (const gchar    *accel_path,
 
 /**
  * gtk_accel_map_lookup_entry:
- * @accel_path:  a valid accelerator path
- * @key:         the accelerator key to be filled in (optional)
- * @returns:     %TRUE if @accel_path is known, %FALSE otherwise
+ * @accel_path: a valid accelerator path
+ * @key: (allow-none) (out): the accelerator key to be filled in (optional)
+ * @returns:    %TRUE if @accel_path is known, %FALSE otherwise
  *
  * Looks up the accelerator entry for @accel_path and fills in @key.
  */
@@ -316,7 +318,7 @@ internal_change_entry (const gchar    *accel_path,
     {
       GtkAccelGroup *group = slist->data;
 
-      for (node = group->acceleratables; node; node = node->next)
+      for (node = _gtk_accel_group_get_accelerables (group); node; node = node->next)
 	g_hash_table_insert (window_hm, node->data, node->data);
     }
   g_slist_free (group_list);
@@ -364,7 +366,7 @@ internal_change_entry (const gchar    *accel_path,
 	for (i = 0; i < n; i++)
 	  {
 	    seen_accel = TRUE;
-	    removable = !group->lock_count && !(ag_entry[i].key.accel_flags & GTK_ACCEL_LOCKED);
+	    removable = !gtk_accel_group_get_is_locked (group) && !(ag_entry[i].key.accel_flags & GTK_ACCEL_LOCKED);
 	    if (!removable)
 	      goto break_loop_step5;
 	    if (ag_entry[i].accel_path_quark)
@@ -430,6 +432,10 @@ internal_change_entry (const gchar    *accel_path,
  * conflicts. A change will only occur if all conflicts could be resolved (which
  * might not be the case if conflicting accelerators are locked). Successful
  * changes are indicated by a %TRUE return value.
+ * 
+ * Note that @accel_path string will be stored in a #GQuark. Therefore, if you
+ * pass a static string, you can save some memory by interning it first with 
+ * g_intern_static_string().
  */
 gboolean
 gtk_accel_map_change_entry (const gchar    *accel_path,
@@ -542,7 +548,7 @@ gtk_accel_map_load_scanner (GScanner *scanner)
   gchar *cpair_comment_single;
   gpointer saved_symbol;
   
-  g_return_if_fail (scanner != 0);
+  g_return_if_fail (scanner != NULL);
 
   /* configure scanner */
   skip_comment_single = scanner->config->skip_comment_single;
@@ -931,7 +937,7 @@ gtk_accel_map_unlock_path (const gchar *accel_path)
   entry->lock_count -= 1;  
 }
 
-G_DEFINE_TYPE (GtkAccelMap, gtk_accel_map, G_TYPE_OBJECT);
+G_DEFINE_TYPE (GtkAccelMap, gtk_accel_map, G_TYPE_OBJECT)
 
 static void
 gtk_accel_map_class_init (GtkAccelMapClass *accel_map_class)
@@ -973,7 +979,7 @@ gtk_accel_map_init (GtkAccelMap *accel_map)
  * map via the ::changed signal; it isn't a parameter to the
  * other accelerator map functions.
  * 
- * Return value: the global #GtkAccelMap object
+ * Return value: (transfer none): the global #GtkAccelMap object
  *
  * Since: 2.4
  **/
@@ -998,7 +1004,7 @@ do_accel_map_changed (AccelEntry *entry)
 		   entry->accel_mods);
 }
 
-#ifdef G_OS_WIN32
+#if defined (G_OS_WIN32) && !defined (_WIN64)
 
 #undef gtk_accel_map_load
 
@@ -1025,6 +1031,3 @@ gtk_accel_map_save (const gchar *file_name)
 }
 
 #endif
-
-#define __GTK_ACCEL_MAP_C__
-#include "gtkaliasdef.c"
