@@ -37,7 +37,8 @@ G_BEGIN_DECLS
 #define GTK_IS_CELL_RENDERER_PIXBUF_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_CELL_RENDERER_PIXBUF))
 #define GTK_CELL_RENDERER_PIXBUF_GET_CLASS(obj)         (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_CELL_RENDERER_PIXBUF, GtkCellRendererPixbufClass))
 
-typedef struct _GtkCellRendererPixbuf GtkCellRendererPixbuf;
+typedef struct _GtkCellRendererPixbuf      GtkCellRendererPixbuf;
+typedef struct _GtkCellRendererPixbufPriv  GtkCellRendererPixbufPriv;
 typedef struct _GtkCellRendererPixbufClass GtkCellRendererPixbufClass;
 
 struct _GtkCellRendererPixbuf
@@ -45,9 +46,7 @@ struct _GtkCellRendererPixbuf
   GtkCellRenderer parent;
 
   /*< private >*/
-  GdkPixbuf *GSEAL (pixbuf);
-  GdkPixbuf *GSEAL (pixbuf_expander_open);
-  GdkPixbuf *GSEAL (pixbuf_expander_closed);
+  GtkCellRendererPixbufPriv *priv;
 };
 
 struct _GtkCellRendererPixbufClass
