@@ -32,7 +32,16 @@
 #include "gtkmarshalers.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
-#include "gtkalias.h"
+
+
+
+struct _GtkCheckMenuItemPriv
+{
+  guint active             : 1;
+  guint always_show_toggle : 1;
+  guint draw_as_radio      : 1;
+  guint inconsistent       : 1;
+};
 
 enum {
   TOGGLED,
@@ -65,10 +74,10 @@ static void gtk_check_menu_item_get_property         (GObject               *obj
 						      GParamSpec            *pspec);
 
 static void gtk_check_menu_item_activatable_interface_init (GtkActivatableIface  *iface);
-static void gtk_check_menu_item_activatable_update         (GtkActivatable       *activatable,
+static void gtk_check_menu_item_update                     (GtkActivatable       *activatable,
 							    GtkAction            *action,
 							    const gchar          *property_name);
-static void gtk_check_menu_item_activatable_reset          (GtkActivatable       *activatable,
+static void gtk_check_menu_item_sync_action_properties     (GtkActivatable       *activatable,
 							    GtkAction            *action);
 
 static GtkActivatableIface *parent_activatable_iface;
@@ -142,20 +151,22 @@ gtk_check_menu_item_class_init (GtkCheckMenuItemClass *klass)
 		  NULL, NULL,
 		  _gtk_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
+
+  g_type_class_add_private (klass, sizeof (GtkCheckMenuItemPriv));
 }
 
 static void 
 gtk_check_menu_item_activatable_interface_init (GtkActivatableIface  *iface)
 {
   parent_activatable_iface = g_type_interface_peek_parent (iface);
-  iface->update = gtk_check_menu_item_activatable_update;
-  iface->reset = gtk_check_menu_item_activatable_reset;
+  iface->update = gtk_check_menu_item_update;
+  iface->sync_action_properties = gtk_check_menu_item_sync_action_properties;
 }
 
-static void 
-gtk_check_menu_item_activatable_update (GtkActivatable       *activatable,
-					GtkAction            *action,
-					const gchar          *property_name)
+static void
+gtk_check_menu_item_update (GtkActivatable *activatable,
+			    GtkAction      *action,
+			    const gchar    *property_name)
 {
   GtkCheckMenuItem *check_menu_item;
 
@@ -174,21 +185,21 @@ gtk_check_menu_item_activatable_update (GtkActivatable       *activatable,
     return;
 
   if (strcmp (property_name, "draw-as-radio") == 0)
-    gtk_check_menu_item_set_draw_as_radio (check_menu_item, 
+    gtk_check_menu_item_set_draw_as_radio (check_menu_item,
 					   gtk_toggle_action_get_draw_as_radio (GTK_TOGGLE_ACTION (action)));
 }
 
-static void 
-gtk_check_menu_item_activatable_reset (GtkActivatable       *activatable,
-				       GtkAction            *action)
+static void
+gtk_check_menu_item_sync_action_properties (GtkActivatable *activatable,
+		                            GtkAction      *action)
 {
   GtkCheckMenuItem *check_menu_item;
 
   check_menu_item = GTK_CHECK_MENU_ITEM (activatable);
 
-  parent_activatable_iface->reset (activatable, action);
+  parent_activatable_iface->sync_action_properties (activatable, action);
 
-  if (!action)
+  if (!GTK_IS_TOGGLE_ACTION (action))
     return;
 
   gtk_action_block_activate (action);
@@ -198,7 +209,7 @@ gtk_check_menu_item_activatable_reset (GtkActivatable       *activatable,
   if (!gtk_activatable_get_use_action_appearance (activatable))
     return;
 
-  gtk_check_menu_item_set_draw_as_radio (check_menu_item, 
+  gtk_check_menu_item_set_draw_as_radio (check_menu_item,
 					 gtk_toggle_action_get_draw_as_radio (GTK_TOGGLE_ACTION (action)));
 }
 
@@ -240,11 +251,15 @@ void
 gtk_check_menu_item_set_active (GtkCheckMenuItem *check_menu_item,
 				gboolean          is_active)
 {
+  GtkCheckMenuItemPriv *priv;
+
   g_return_if_fail (GTK_IS_CHECK_MENU_ITEM (check_menu_item));
 
+  priv = check_menu_item->priv;
+
   is_active = is_active != 0;
 
-  if (check_menu_item->active != is_active)
+  if (priv->active != is_active)
     gtk_menu_item_activate (GTK_MENU_ITEM (check_menu_item));
 }
 
@@ -262,7 +277,7 @@ gtk_check_menu_item_get_active (GtkCheckMenuItem *check_menu_item)
 {
   g_return_val_if_fail (GTK_IS_CHECK_MENU_ITEM (check_menu_item), FALSE);
 
-  return check_menu_item->active;
+  return check_menu_item->priv->active;
 }
 
 static void
@@ -282,17 +297,6 @@ gtk_check_menu_item_toggle_size_request (GtkMenuItem *menu_item,
   *requisition = indicator_size + toggle_spacing;
 }
 
-void
-gtk_check_menu_item_set_show_toggle (GtkCheckMenuItem *menu_item,
-				     gboolean          always)
-{
-  g_return_if_fail (GTK_IS_CHECK_MENU_ITEM (menu_item));
-
-#if 0
-  menu_item->always_show_toggle = always != FALSE;
-#endif  
-}
-
 void
 gtk_check_menu_item_toggled (GtkCheckMenuItem *check_menu_item)
 {
@@ -318,13 +322,17 @@ void
 gtk_check_menu_item_set_inconsistent (GtkCheckMenuItem *check_menu_item,
                                       gboolean          setting)
 {
+  GtkCheckMenuItemPriv *priv;
+
   g_return_if_fail (GTK_IS_CHECK_MENU_ITEM (check_menu_item));
+
+  priv = check_menu_item->priv;
   
   setting = setting != FALSE;
 
-  if (setting != check_menu_item->inconsistent)
+  if (setting != priv->inconsistent)
     {
-      check_menu_item->inconsistent = setting;
+      priv->inconsistent = setting;
       gtk_widget_queue_draw (GTK_WIDGET (check_menu_item));
       g_object_notify (G_OBJECT (check_menu_item), "inconsistent");
     }
@@ -343,7 +351,7 @@ gtk_check_menu_item_get_inconsistent (GtkCheckMenuItem *check_menu_item)
 {
   g_return_val_if_fail (GTK_IS_CHECK_MENU_ITEM (check_menu_item), FALSE);
 
-  return check_menu_item->inconsistent;
+  return check_menu_item->priv->inconsistent;
 }
 
 /**
@@ -359,13 +367,17 @@ void
 gtk_check_menu_item_set_draw_as_radio (GtkCheckMenuItem *check_menu_item,
 				       gboolean          draw_as_radio)
 {
+  GtkCheckMenuItemPriv *priv;
+
   g_return_if_fail (GTK_IS_CHECK_MENU_ITEM (check_menu_item));
-  
+
+  priv = check_menu_item->priv;
+
   draw_as_radio = draw_as_radio != FALSE;
 
-  if (draw_as_radio != check_menu_item->draw_as_radio)
+  if (draw_as_radio != priv->draw_as_radio)
     {
-      check_menu_item->draw_as_radio = draw_as_radio;
+      priv->draw_as_radio = draw_as_radio;
 
       gtk_widget_queue_draw (GTK_WIDGET (check_menu_item));
 
@@ -388,14 +400,21 @@ gtk_check_menu_item_get_draw_as_radio (GtkCheckMenuItem *check_menu_item)
 {
   g_return_val_if_fail (GTK_IS_CHECK_MENU_ITEM (check_menu_item), FALSE);
   
-  return check_menu_item->draw_as_radio;
+  return check_menu_item->priv->draw_as_radio;
 }
 
 static void
 gtk_check_menu_item_init (GtkCheckMenuItem *check_menu_item)
 {
-  check_menu_item->active = FALSE;
-  check_menu_item->always_show_toggle = TRUE;
+  GtkCheckMenuItemPriv *priv;
+
+  check_menu_item->priv = G_TYPE_INSTANCE_GET_PRIVATE (check_menu_item,
+                                                       GTK_TYPE_CHECK_MENU_ITEM,
+                                                       GtkCheckMenuItemPriv);
+  priv = check_menu_item->priv; 
+
+  priv->active = FALSE;
+  priv->always_show_toggle = TRUE;
 }
 
 static gint
@@ -413,8 +432,12 @@ gtk_check_menu_item_expose (GtkWidget      *widget,
 static void
 gtk_check_menu_item_activate (GtkMenuItem *menu_item)
 {
+  GtkCheckMenuItemPriv *priv;
+
   GtkCheckMenuItem *check_menu_item = GTK_CHECK_MENU_ITEM (menu_item);
-  check_menu_item->active = !check_menu_item->active;
+  priv = check_menu_item->priv;
+
+  priv->active = !priv->active;
 
   gtk_check_menu_item_toggled (check_menu_item);
   gtk_widget_queue_draw (GTK_WIDGET (check_menu_item));
@@ -436,30 +459,32 @@ static void
 gtk_real_check_menu_item_draw_indicator (GtkCheckMenuItem *check_menu_item,
 					 GdkRectangle     *area)
 {
+  GtkCheckMenuItemPriv *priv = check_menu_item->priv;
   GtkWidget *widget;
   GtkStateType state_type;
   GtkShadowType shadow_type;
   gint x, y;
 
-  if (GTK_WIDGET_DRAWABLE (check_menu_item))
+  widget = GTK_WIDGET (check_menu_item);
+
+  if (gtk_widget_is_drawable (widget))
     {
+      guint border_width;
       guint offset;
       guint toggle_size;
       guint toggle_spacing;
       guint horizontal_padding;
       guint indicator_size;
-      
-      widget = GTK_WIDGET (check_menu_item);
 
-      gtk_widget_style_get (GTK_WIDGET (check_menu_item),
+      gtk_widget_style_get (widget,
  			    "toggle-spacing", &toggle_spacing,
  			    "horizontal-padding", &horizontal_padding,
 			    "indicator-size", &indicator_size,
  			    NULL);
 
       toggle_size = GTK_MENU_ITEM (check_menu_item)->toggle_size;
-      offset = GTK_CONTAINER (check_menu_item)->border_width +
-	widget->style->xthickness + 2; 
+      border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
+      offset = border_width + widget->style->xthickness + 2;
 
       if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR)
 	{
@@ -475,23 +500,23 @@ gtk_real_check_menu_item_draw_indicator (GtkCheckMenuItem *check_menu_item,
       
       y = widget->allocation.y + (widget->allocation.height - indicator_size) / 2;
 
-      if (check_menu_item->active ||
-	  check_menu_item->always_show_toggle ||
-	  (GTK_WIDGET_STATE (check_menu_item) == GTK_STATE_PRELIGHT))
+      if (priv->active ||
+	  priv->always_show_toggle ||
+	  (gtk_widget_get_state (widget) == GTK_STATE_PRELIGHT))
 	{
-	  state_type = GTK_WIDGET_STATE (widget);
+	  state_type = gtk_widget_get_state (widget);
 	  
-	  if (check_menu_item->inconsistent)
+	  if (priv->inconsistent)
 	    shadow_type = GTK_SHADOW_ETCHED_IN;
-	  else if (check_menu_item->active)
+	  else if (priv->active)
 	    shadow_type = GTK_SHADOW_IN;
 	  else 
 	    shadow_type = GTK_SHADOW_OUT;
 	  
-	  if (!GTK_WIDGET_IS_SENSITIVE (widget))
+	  if (!gtk_widget_is_sensitive (widget))
 	    state_type = GTK_STATE_INSENSITIVE;
 
-	  if (check_menu_item->draw_as_radio)
+	  if (priv->draw_as_radio)
 	    {
 	      gtk_paint_option (widget->style, widget->window,
 				state_type, shadow_type,
@@ -517,17 +542,18 @@ gtk_check_menu_item_get_property (GObject     *object,
 				  GParamSpec  *pspec)
 {
   GtkCheckMenuItem *checkitem = GTK_CHECK_MENU_ITEM (object);
+  GtkCheckMenuItemPriv *priv = checkitem->priv;
   
   switch (prop_id)
     {
     case PROP_ACTIVE:
-      g_value_set_boolean (value, checkitem->active);
+      g_value_set_boolean (value, priv->active);
       break;
     case PROP_INCONSISTENT:
-      g_value_set_boolean (value, checkitem->inconsistent);
+      g_value_set_boolean (value, priv->inconsistent);
       break;
     case PROP_DRAW_AS_RADIO:
-      g_value_set_boolean (value, checkitem->draw_as_radio);
+      g_value_set_boolean (value, priv->draw_as_radio);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -561,5 +587,22 @@ gtk_check_menu_item_set_property (GObject      *object,
     }
 }
 
-#define __GTK_CHECK_MENU_ITEM_C__
-#include "gtkaliasdef.c"
+
+/* Private */
+
+/*
+ * _gtk_check_menu_item_set_active:
+ * @check_menu_item: a #GtkCheckMenuItem
+ * @is_active: whether the action is active or not
+ *
+ * Sets the #GtkCheckMenuItem:active property directly. This function does
+ * not emit signals or notifications: it is left to the caller to do so.
+ */
+void
+_gtk_check_menu_item_set_active (GtkCheckMenuItem *check_menu_item,
+                                 gboolean          is_active)
+{
+  GtkCheckMenuItemPriv *priv = check_menu_item->priv;
+
+  priv->active = is_active;
+}
