@@ -44,20 +44,15 @@
 #include "gtkmarshalers.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
-#include "gtkalias.h"
 
 /* Size of checks and gray levels for alpha compositing checkerboard */
 #define CHECK_SIZE  4
-#define CHECK_DARK  21845  /* 65535 / 3     */
-#define CHECK_LIGHT 43690 
+#define CHECK_DARK  (1.0 / 3.0)
+#define CHECK_LIGHT (2.0 / 3.0)
 
-#define GTK_COLOR_BUTTON_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GTK_TYPE_COLOR_BUTTON, GtkColorButtonPrivate))
 
 struct _GtkColorButtonPrivate 
 {
-  GdkPixbuf *pixbuf;    /* Pixbuf for rendering sample */
-  GdkGC *gc;            /* GC for drawing */
-  
   GtkWidget *draw_area; /* Widget where we draw the color sample */
   GtkWidget *cs_dialog; /* Color selection dialog */
   
@@ -98,12 +93,8 @@ static void gtk_color_button_get_property  (GObject        *object,
 					    GParamSpec     *pspec);
 
 /* gtkwidget signals */
-static void gtk_color_button_realize       (GtkWidget *widget);
-static void gtk_color_button_unrealize     (GtkWidget *widget);
 static void gtk_color_button_state_changed (GtkWidget           *widget, 
 					    GtkStateType         previous_state);
-static void gtk_color_button_style_set     (GtkWidget *widget, 
-					    GtkStyle  *previous_style);
 
 /* gtkbutton signals */
 static void gtk_color_button_clicked       (GtkButton           *button);
@@ -151,9 +142,6 @@ gtk_color_button_class_init (GtkColorButtonClass *klass)
   gobject_class->set_property = gtk_color_button_set_property;
   gobject_class->finalize = gtk_color_button_finalize;
   widget_class->state_changed = gtk_color_button_state_changed;
-  widget_class->realize = gtk_color_button_realize;
-  widget_class->unrealize = gtk_color_button_unrealize;
-  widget_class->style_set = gtk_color_button_style_set;
   button_class->clicked = gtk_color_button_clicked;
   klass->color_set = NULL;
 
@@ -169,7 +157,7 @@ gtk_color_button_class_init (GtkColorButtonClass *klass)
   g_object_class_install_property (gobject_class,
                                    PROP_USE_ALPHA,
                                    g_param_spec_boolean ("use-alpha", P_("Use alpha"), 
-                                                         P_("Whether or not to give the color an alpha value"),
+                                                         P_("Whether to give the color an alpha value"),
                                                          FALSE,
                                                          GTK_PARAM_READWRITE));
 
@@ -232,7 +220,7 @@ gtk_color_button_class_init (GtkColorButtonClass *klass)
    *
    * Since: 2.4
    */
-  color_button_signals[COLOR_SET] = g_signal_new (I_("color_set"),
+  color_button_signals[COLOR_SET] = g_signal_new (I_("color-set"),
 						  G_TYPE_FROM_CLASS (gobject_class),
 						  G_SIGNAL_RUN_FIRST,
 						  G_STRUCT_OFFSET (GtkColorButtonClass, color_set),
@@ -243,102 +231,34 @@ gtk_color_button_class_init (GtkColorButtonClass *klass)
   g_type_class_add_private (gobject_class, sizeof (GtkColorButtonPrivate));
 }
 
-static void
-render (GtkColorButton *color_button)
+static gboolean
+gtk_color_button_has_alpha (GtkColorButton *color_button)
 {
-  gint dark_r, dark_g, dark_b;
-  gint light_r, light_g, light_b;
-  gint i, j, rowstride;
-  gint width, height;
-  gint c1[3], c2[3];
-  guchar *pixels;
-  guint8 insensitive_r = 0;
-  guint8 insensitive_g = 0;
-  guint8 insensitive_b = 0;
+  return color_button->priv->use_alpha &&
+      color_button->priv->alpha < 65535;
+}
 
-  width = color_button->priv->draw_area->allocation.width;
-  height = color_button->priv->draw_area->allocation.height;
-  if (color_button->priv->pixbuf == NULL || 
-      gdk_pixbuf_get_width (color_button->priv->pixbuf) != width ||
-      gdk_pixbuf_get_height (color_button->priv->pixbuf) != height) 
+static cairo_pattern_t *
+gtk_color_button_get_checkered (void)
+{
+  /* need to respect pixman's stride being a multiple of 4 */
+  static unsigned char data[8] = { 0xFF, 0x00, 0x00, 0x00,
+                                   0x00, 0xFF, 0x00, 0x00 };
+  static cairo_surface_t *checkered = NULL;
+  cairo_pattern_t *pattern;
+
+  if (checkered == NULL)
     {
-      if (color_button->priv->pixbuf != NULL)
-	g_object_unref (color_button->priv->pixbuf);
-      color_button->priv->pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, width, height);
-    }
-  
-
-  /* Compute dark and light check colors */
-
-  insensitive_r = GTK_WIDGET(color_button)->style->bg[GTK_STATE_INSENSITIVE].red >> 8;
-  insensitive_g = GTK_WIDGET(color_button)->style->bg[GTK_STATE_INSENSITIVE].green >> 8;
-  insensitive_b = GTK_WIDGET(color_button)->style->bg[GTK_STATE_INSENSITIVE].blue >> 8;
-
-  if (color_button->priv->use_alpha) 
-    {
-      dark_r = ((CHECK_DARK << 16) + (color_button->priv->color.red - CHECK_DARK) * color_button->priv->alpha) >> 24;
-      dark_g = ((CHECK_DARK << 16) + (color_button->priv->color.green - CHECK_DARK) * color_button->priv->alpha) >> 24;
-      dark_b = ((CHECK_DARK << 16) + (color_button->priv->color.blue - CHECK_DARK) * color_button->priv->alpha) >> 24;
-      
-      light_r = ((CHECK_LIGHT << 16) + (color_button->priv->color.red - CHECK_LIGHT) * color_button->priv->alpha) >> 24;
-      light_g = ((CHECK_LIGHT << 16) + (color_button->priv->color.green - CHECK_LIGHT) * color_button->priv->alpha) >> 24;
-      light_b = ((CHECK_LIGHT << 16) + (color_button->priv->color.blue - CHECK_LIGHT) * color_button->priv->alpha) >> 24;
-    } 
-  else 
-    {
-      dark_r = light_r = color_button->priv->color.red >> 8;
-      dark_g = light_g = color_button->priv->color.green >> 8;
-      dark_b = light_b = color_button->priv->color.blue >> 8;
+      checkered = cairo_image_surface_create_for_data (data,
+                                                       CAIRO_FORMAT_A8,
+                                                       2, 2, 4);
     }
 
-  /* Fill image buffer */
-  pixels = gdk_pixbuf_get_pixels (color_button->priv->pixbuf);
-  rowstride = gdk_pixbuf_get_rowstride (color_button->priv->pixbuf);
-  for (j = 0; j < height; j++) 
-    {
-      if ((j / CHECK_SIZE) & 1) 
-        {
-          c1[0] = dark_r;
-          c1[1] = dark_g;
-          c1[2] = dark_b;
-          
-          c2[0] = light_r;
-          c2[1] = light_g;
-          c2[2] = light_b;
-        } 
-      else 
-        {
-          c1[0] = light_r;
-          c1[1] = light_g;
-          c1[2] = light_b;
-          
-          c2[0] = dark_r;
-          c2[1] = dark_g;
-          c2[2] = dark_b;
-        }
-      
-    for (i = 0; i < width; i++) 
-      {
-        if (!GTK_WIDGET_IS_SENSITIVE (GTK_WIDGET (color_button)) && (i+j)%2) 
-          {
-            *(pixels + j * rowstride + i * 3) = insensitive_r;
-            *(pixels + j * rowstride + i * 3 + 1) = insensitive_g;
-            *(pixels + j * rowstride + i * 3 + 2) = insensitive_b;
-          } 
-        else if ((i / CHECK_SIZE) & 1) 
-          {
-            *(pixels + j * rowstride + i * 3)     = c1[0];
-            *(pixels + j * rowstride + i * 3 + 1) = c1[1];
-            *(pixels + j * rowstride + i * 3 + 2) = c1[2];
-          }
-        else
-          {
-            *(pixels + j * rowstride + i * 3)     = c2[0];
-            *(pixels + j * rowstride + i * 3 + 1) = c2[1];
-            *(pixels + j * rowstride + i * 3 + 2) = c2[2];
-          }
-      }
-    }
+  pattern = cairo_pattern_create_for_surface (checkered);
+  cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
+  cairo_pattern_set_filter (pattern, CAIRO_FILTER_NEAREST);
+
+  return pattern;
 }
 
 /* Handle exposure events for the color picker's drawing area */
@@ -348,81 +268,63 @@ expose_event (GtkWidget      *widget,
               gpointer        data)
 {
   GtkColorButton *color_button = GTK_COLOR_BUTTON (data);
+  GtkAllocation allocation;
+  cairo_pattern_t *checkered;
+  cairo_t *cr;
 
-  gint width = color_button->priv->draw_area->allocation.width;
-  gint height = color_button->priv->draw_area->allocation.height;
+  cr = gdk_cairo_create (event->window);
 
-  if (color_button->priv->pixbuf == NULL ||
-      width != gdk_pixbuf_get_width (color_button->priv->pixbuf) ||
-      height != gdk_pixbuf_get_height (color_button->priv->pixbuf)) 
-    render (color_button);
+  gtk_widget_get_allocation (widget, &allocation);
+  gdk_cairo_rectangle (cr, &allocation);
+  cairo_clip (cr);
+
+  if (gtk_color_button_has_alpha (color_button))
+    {
+      cairo_save (cr);
+
+      cairo_set_source_rgb (cr, CHECK_DARK, CHECK_DARK, CHECK_DARK);
+      cairo_paint (cr);
+
+      cairo_set_source_rgb (cr, CHECK_LIGHT, CHECK_LIGHT, CHECK_LIGHT);
+      cairo_scale (cr, CHECK_SIZE, CHECK_SIZE);
+
+      checkered = gtk_color_button_get_checkered ();
+      cairo_mask (cr, checkered);
+      cairo_pattern_destroy (checkered);
+
+      cairo_restore (cr);
+
+      cairo_set_source_rgba (cr,
+                             color_button->priv->color.red / 65535.,
+                             color_button->priv->color.green / 65535.,
+                             color_button->priv->color.blue / 65535.,
+                             color_button->priv->alpha / 65535.);
+    }
+  else
+    {
+      gdk_cairo_set_source_color (cr, &color_button->priv->color);
+    }
+
+  cairo_paint (cr);
+
+  if (!gtk_widget_is_sensitive (GTK_WIDGET (color_button)))
+    {
+      gdk_cairo_set_source_color (cr, &GTK_WIDGET(color_button)->style->bg[GTK_STATE_INSENSITIVE]);
+      checkered = gtk_color_button_get_checkered ();
+      cairo_mask (cr, checkered);
+      cairo_pattern_destroy (checkered);
+    }
+
+  cairo_destroy (cr);
 
-  gdk_draw_pixbuf (widget->window,
-                   color_button->priv->gc,
-                   color_button->priv->pixbuf,
-                   event->area.x - widget->allocation.x,
-                   event->area.y - widget->allocation.y,
-                   event->area.x,
-                   event->area.y,
-                   event->area.width,
-                   event->area.height,
-                   GDK_RGB_DITHER_MAX,
-                   event->area.x - widget->allocation.x,
-                   event->area.y - widget->allocation.y);
   return FALSE;
 }
 
-static void
-gtk_color_button_realize (GtkWidget *widget)
-{
-  GtkColorButton *color_button = GTK_COLOR_BUTTON (widget);
-
-  GTK_WIDGET_CLASS (gtk_color_button_parent_class)->realize (widget);
-
-  color_button->priv->gc = gdk_gc_new (widget->window);
-
-  render (color_button);
-}
-
-static void
-gtk_color_button_unrealize (GtkWidget *widget)
-{
-  GtkColorButton *color_button = GTK_COLOR_BUTTON (widget);
-
-  g_object_unref (color_button->priv->gc);
-  color_button->priv->gc = NULL;
-
-  GTK_WIDGET_CLASS (gtk_color_button_parent_class)->unrealize (widget);
-}
-
-static void
-gtk_color_button_style_set (GtkWidget *widget, 
-			    GtkStyle  *previous_style)
-{
-  GtkColorButton *color_button = GTK_COLOR_BUTTON (widget);
-
-  GTK_WIDGET_CLASS (gtk_color_button_parent_class)->style_set (widget, previous_style);
-
-  if (GTK_WIDGET_REALIZED (widget)) 
-    {
-      if (color_button->priv->pixbuf != NULL)
-	g_object_unref (color_button->priv->pixbuf);
-      color_button->priv->pixbuf = NULL;
-    }
-}
-
 static void
 gtk_color_button_state_changed (GtkWidget   *widget, 
-                                          GtkStateType previous_state)
+                                GtkStateType previous_state)
 {
-  GtkColorButton *color_button = GTK_COLOR_BUTTON (widget);
-
-  if (widget->state == GTK_STATE_INSENSITIVE || previous_state == GTK_STATE_INSENSITIVE)
-    {
-      if (color_button->priv->pixbuf != NULL)
-	g_object_unref (color_button->priv->pixbuf);
-      color_button->priv->pixbuf = NULL;
-    }
+  gtk_widget_queue_draw (widget);
 }
 
 static void
@@ -457,10 +359,6 @@ gtk_color_button_drag_data_received (GtkWidget        *widget,
   color_button->priv->color.blue = dropped[2];
   color_button->priv->alpha = dropped[3];
 
-  if (color_button->priv->pixbuf != NULL)
-    g_object_unref (color_button->priv->pixbuf);
-  color_button->priv->pixbuf = NULL;
-
   gtk_widget_queue_draw (color_button->priv->draw_area);
 
   g_signal_emit (color_button, color_button_signals[COLOR_SET], 0);
@@ -529,7 +427,9 @@ gtk_color_button_init (GtkColorButton *color_button)
   PangoRectangle rect;
 
   /* Create the widgets */
-  color_button->priv = GTK_COLOR_BUTTON_GET_PRIVATE (color_button);
+  color_button->priv = G_TYPE_INSTANCE_GET_PRIVATE (color_button,
+                                                    GTK_TYPE_COLOR_BUTTON,
+                                                    GtkColorButtonPrivate);
 
   gtk_widget_push_composite_child ();
 
@@ -558,13 +458,6 @@ gtk_color_button_init (GtkColorButton *color_button)
 
   color_button->priv->title = g_strdup (_("Pick a Color")); /* default title */
 
-  /* Create the buffer for the image so that we can create an image.  
-   * Also create the picker's pixmap.
-   */
-  color_button->priv->pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, rect.width, rect.height);
-
-  color_button->priv->gc = NULL;
-
   /* Start with opaque black, alpha disabled */
 
   color_button->priv->color.red = 0;
@@ -582,11 +475,11 @@ gtk_color_button_init (GtkColorButton *color_button)
                        GDK_BUTTON1_MASK|GDK_BUTTON3_MASK,
                        drop_types, 1,
                        GDK_ACTION_COPY);
-  g_signal_connect (color_button, "drag_begin",
+  g_signal_connect (color_button, "drag-begin",
 		    G_CALLBACK (gtk_color_button_drag_begin), color_button);
-  g_signal_connect (color_button, "drag_data_received",
+  g_signal_connect (color_button, "drag-data-received",
                     G_CALLBACK (gtk_color_button_drag_data_received), color_button);
-  g_signal_connect (color_button, "drag_data_get",
+  g_signal_connect (color_button, "drag-data-get",
                     G_CALLBACK (gtk_color_button_drag_data_get), color_button);
 
   gtk_widget_pop_composite_child ();
@@ -601,10 +494,6 @@ gtk_color_button_finalize (GObject *object)
     gtk_widget_destroy (color_button->priv->cs_dialog);
   color_button->priv->cs_dialog = NULL;
 
-  if (color_button->priv->pixbuf != NULL)
-    g_object_unref (color_button->priv->pixbuf);
-  color_button->priv->pixbuf = NULL;
-
   g_free (color_button->priv->title);
   color_button->priv->title = NULL;
 
@@ -653,16 +542,14 @@ dialog_ok_clicked (GtkWidget *widget,
 {
   GtkColorButton *color_button = GTK_COLOR_BUTTON (data);
   GtkColorSelection *color_selection;
+  GtkColorSelectionDialog *selection_dialog;
 
-  color_selection = GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (color_button->priv->cs_dialog)->colorsel);
+  selection_dialog = GTK_COLOR_SELECTION_DIALOG (color_button->priv->cs_dialog);
+  color_selection = GTK_COLOR_SELECTION (gtk_color_selection_dialog_get_color_selection (selection_dialog));
 
   gtk_color_selection_get_current_color (color_selection, &color_button->priv->color);
   color_button->priv->alpha = gtk_color_selection_get_current_alpha (color_selection);
 
-  if (color_button->priv->pixbuf != NULL)
-    g_object_unref (color_button->priv->pixbuf);
-  color_button->priv->pixbuf = NULL;
-
   gtk_widget_hide (color_button->priv->cs_dialog);
 
   gtk_widget_queue_draw (color_button->priv->draw_area);
@@ -699,6 +586,7 @@ static void
 gtk_color_button_clicked (GtkButton *button)
 {
   GtkColorButton *color_button = GTK_COLOR_BUTTON (button);
+  GtkColorSelection *color_selection;
   GtkColorSelectionDialog *color_dialog;
 
   /* if dialog already exists, make sure it's shown and raised */
@@ -706,6 +594,7 @@ gtk_color_button_clicked (GtkButton *button)
     {
       /* Create the dialog and connects its buttons */
       GtkWidget *parent;
+      GtkWidget *ok_button, *cancel_button;
       
       parent = gtk_widget_get_toplevel (GTK_WIDGET (color_button));
       
@@ -713,7 +602,7 @@ gtk_color_button_clicked (GtkButton *button)
       
       color_dialog = GTK_COLOR_SELECTION_DIALOG (color_button->priv->cs_dialog);
 
-      if (GTK_WIDGET_TOPLEVEL (parent) && GTK_IS_WINDOW (parent))
+      if (gtk_widget_is_toplevel (parent) && GTK_IS_WINDOW (parent))
         {
           if (GTK_WINDOW (parent) != gtk_window_get_transient_for (GTK_WINDOW (color_dialog)))
  	    gtk_window_set_transient_for (GTK_WINDOW (color_dialog), GTK_WINDOW (parent));
@@ -721,28 +610,34 @@ gtk_color_button_clicked (GtkButton *button)
 	  gtk_window_set_modal (GTK_WINDOW (color_dialog),
 				gtk_window_get_modal (GTK_WINDOW (parent)));
 	}
+
+      g_object_get (color_dialog,
+                    "ok-button", &ok_button,
+                    "cancel-button", &cancel_button,
+                    NULL);
       
-      g_signal_connect (color_dialog->ok_button, "clicked",
+      g_signal_connect (ok_button, "clicked",
                         G_CALLBACK (dialog_ok_clicked), color_button);
-      g_signal_connect (color_dialog->cancel_button, "clicked",
+      g_signal_connect (cancel_button, "clicked",
 			G_CALLBACK (dialog_cancel_clicked), color_button);
       g_signal_connect (color_dialog, "destroy",
                         G_CALLBACK (dialog_destroy), color_button);
     }
 
   color_dialog = GTK_COLOR_SELECTION_DIALOG (color_button->priv->cs_dialog);
+  color_selection = GTK_COLOR_SELECTION (gtk_color_selection_dialog_get_color_selection (color_dialog));
 
-  gtk_color_selection_set_has_opacity_control (GTK_COLOR_SELECTION (color_dialog->colorsel),
+  gtk_color_selection_set_has_opacity_control (color_selection,
                                                color_button->priv->use_alpha);
   
-  gtk_color_selection_set_previous_color (GTK_COLOR_SELECTION (color_dialog->colorsel), 
+  gtk_color_selection_set_previous_color (color_selection,
 					  &color_button->priv->color);
-  gtk_color_selection_set_previous_alpha (GTK_COLOR_SELECTION (color_dialog->colorsel), 
+  gtk_color_selection_set_previous_alpha (color_selection,
 					  color_button->priv->alpha);
 
-  gtk_color_selection_set_current_color (GTK_COLOR_SELECTION (color_dialog->colorsel), 
+  gtk_color_selection_set_current_color (color_selection,
 					 &color_button->priv->color);
-  gtk_color_selection_set_current_alpha (GTK_COLOR_SELECTION (color_dialog->colorsel), 
+  gtk_color_selection_set_current_alpha (color_selection,
 					 color_button->priv->alpha);
 
   gtk_window_present (GTK_WINDOW (color_button->priv->cs_dialog));
@@ -768,10 +663,6 @@ gtk_color_button_set_color (GtkColorButton *color_button,
   color_button->priv->color.green = color->green;
   color_button->priv->color.blue = color->blue;
 
-  if (color_button->priv->pixbuf != NULL)
-    g_object_unref (color_button->priv->pixbuf);
-  color_button->priv->pixbuf = NULL;
-
   gtk_widget_queue_draw (color_button->priv->draw_area);
   
   g_object_notify (G_OBJECT (color_button), "color");
@@ -795,10 +686,6 @@ gtk_color_button_set_alpha (GtkColorButton *color_button,
 
   color_button->priv->alpha = alpha;
 
-  if (color_button->priv->pixbuf != NULL)
-    g_object_unref (color_button->priv->pixbuf);
-  color_button->priv->pixbuf = NULL;
-
   gtk_widget_queue_draw (color_button->priv->draw_area);
 
   g_object_notify (G_OBJECT (color_button), "alpha");
@@ -863,7 +750,6 @@ gtk_color_button_set_use_alpha (GtkColorButton *color_button,
     {
       color_button->priv->use_alpha = use_alpha;
 
-      render (color_button);
       gtk_widget_queue_draw (color_button->priv->draw_area);
 
       g_object_notify (G_OBJECT (color_button), "use-alpha");
@@ -992,6 +878,3 @@ gtk_color_button_get_property (GObject    *object,
       break;
     }
 }
-
-#define __GTK_COLOR_BUTTON_C__
-#include "gtkaliasdef.c"
