@@ -24,7 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
 #error "Only <gtk/gtk.h> can be included directly."
 #endif
 
@@ -147,6 +147,24 @@ void               gtk_range_set_inverted                  (GtkRange      *range
                                                             gboolean       setting);
 gboolean           gtk_range_get_inverted                  (GtkRange      *range);
 
+void               gtk_range_set_flippable                 (GtkRange      *range,
+                                                            gboolean       flippable);
+gboolean           gtk_range_get_flippable                 (GtkRange      *range);
+
+void               gtk_range_set_slider_size_fixed         (GtkRange      *range,
+                                                            gboolean       size_fixed);
+gboolean           gtk_range_get_slider_size_fixed         (GtkRange      *range);
+
+void               gtk_range_set_min_slider_size           (GtkRange      *range,
+                                                            gint           min_size);
+gint               gtk_range_get_min_slider_size           (GtkRange      *range);
+
+void               gtk_range_get_range_rect                (GtkRange      *range,
+                                                            GdkRectangle  *range_rect);
+void               gtk_range_get_slider_range              (GtkRange      *range,
+                                                            gint          *slider_start,
+                                                            gint          *slider_end);
+
 void               gtk_range_set_lower_stepper_sensitivity (GtkRange      *range,
                                                             GtkSensitivityType sensitivity);
 GtkSensitivityType gtk_range_get_lower_stepper_sensitivity (GtkRange      *range);
@@ -178,6 +196,12 @@ gdouble            gtk_range_get_fill_level                (GtkRange      *range
 gdouble            _gtk_range_get_wheel_delta              (GtkRange      *range,
                                                             GdkScrollDirection direction);
 
+void               _gtk_range_set_stop_values              (GtkRange      *range,
+                                                            gdouble       *values,
+                                                            gint           n_values);
+gint               _gtk_range_get_stop_positions           (GtkRange      *range,
+                                                            gint         **values);          
+
 
 G_END_DECLS
 
