@@ -1,5 +1,5 @@
 /* GTK - The GIMP Toolkit
- * gtksizegroup.h: 
+ * gtksizegroup.h:
  * Copyright (C) 2000 Red Hat Software
  *
  * This library is free software; you can redistribute it and/or
@@ -18,6 +18,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtk.h> can be included directly."
+#endif
+
 #ifndef __GTK_SIZE_GROUP_H__
 #define __GTK_SIZE_GROUP_H__
 
@@ -34,21 +38,15 @@ G_BEGIN_DECLS
 
 
 typedef struct _GtkSizeGroup       GtkSizeGroup;
+typedef struct _GtkSizeGroupPriv   GtkSizeGroupPriv;
 typedef struct _GtkSizeGroupClass  GtkSizeGroupClass;
 
 struct _GtkSizeGroup
 {
   GObject parent_instance;
 
-  GSList *widgets;
-
-  guint8 mode;
-  
-  guint have_width : 1;
-  guint have_height : 1;
-  guint ignore_hidden : 1;
-
-  GtkRequisition requisition;
+  /* <private> */
+  GtkSizeGroupPriv *priv;
 };
 
 struct _GtkSizeGroupClass
@@ -64,11 +62,11 @@ struct _GtkSizeGroupClass
 
 /**
  * GtkSizeGroupMode:
- * @GTK_SIZE_GROUP_NONE: group has no effect  
+ * @GTK_SIZE_GROUP_NONE: group has no effect
  * @GTK_SIZE_GROUP_HORIZONTAL: group affects horizontal requisition
  * @GTK_SIZE_GROUP_VERTICAL: group affects vertical requisition
  * @GTK_SIZE_GROUP_BOTH: group affects both horizontal and vertical requisition
- * 
+ *
  * The mode of the size group determines the directions in which the size
  * group affects the requested sizes of its component widgets.
  **/
@@ -92,13 +90,15 @@ void             gtk_size_group_add_widget    (GtkSizeGroup     *size_group,
 					       GtkWidget        *widget);
 void             gtk_size_group_remove_widget (GtkSizeGroup     *size_group,
 					       GtkWidget        *widget);
+GSList *         gtk_size_group_get_widgets   (GtkSizeGroup     *size_group);
 
 
-void _gtk_size_group_get_child_requisition (GtkWidget      *widget,
-					    GtkRequisition *requisition);
-void _gtk_size_group_compute_requisition   (GtkWidget      *widget,
-					    GtkRequisition *requisition);
-void _gtk_size_group_queue_resize          (GtkWidget      *widget);
+
+gint _gtk_size_group_bump_requisition      (GtkWidget        *widget,
+					    GtkSizeGroupMode  mode,
+					    gint              widget_requisition);
+void _gtk_size_group_queue_resize          (GtkWidget        *widget);
+
 
 G_END_DECLS
 
