@@ -27,7 +27,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
 #error "Only <gtk/gtk.h> can be included directly."
 #endif
 
@@ -35,7 +35,6 @@
 #define __GTK_SPIN_BUTTON_H__
 
 
-#include <gdk/gdk.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkadjustment.h>
 
@@ -70,6 +69,7 @@ typedef enum
 
 
 typedef struct _GtkSpinButton	    GtkSpinButton;
+typedef struct _GtkSpinButtonPriv   GtkSpinButtonPriv;
 typedef struct _GtkSpinButtonClass  GtkSpinButtonClass;
 
 
@@ -77,26 +77,8 @@ struct _GtkSpinButton
 {
   GtkEntry entry;
 
-  GtkAdjustment *adjustment;
-
-  GdkWindow *panel;
-
-  guint32 timer;
-
-  gdouble climb_rate;
-  gdouble timer_step;
-
-  GtkSpinButtonUpdatePolicy update_policy;
-
-  guint in_child : 2;
-  guint click_child : 2; /* valid: GTK_ARROW_UP=0, GTK_ARROW_DOWN=1 or 2=NONE/BOTH */
-  guint button : 2;
-  guint need_timer : 1;
-  guint timer_calls : 3;
-  guint digits : 10;
-  guint numeric : 1;
-  guint wrap : 1;
-  guint snap_to_ticks : 1;
+  /* <private> */
+  GtkSpinButtonPriv *priv;
 };
 
 struct _GtkSpinButtonClass
@@ -187,10 +169,8 @@ void		gtk_spin_button_set_snap_to_ticks  (GtkSpinButton  *spin_button,
 gboolean        gtk_spin_button_get_snap_to_ticks  (GtkSpinButton  *spin_button);
 void            gtk_spin_button_update             (GtkSpinButton  *spin_button);
 
-
-#ifndef GTK_DISABLE_DEPRECATED
-#define gtk_spin_button_get_value_as_float gtk_spin_button_get_value
-#endif
+/* private */
+GdkWindow*      _gtk_spin_button_get_panel         (GtkSpinButton  *spin_button);
 
 G_END_DECLS
 
