@@ -75,10 +75,9 @@
  */
 
 #define GTK_TEXT_USE_INTERNAL_UNSUPPORTED_API
-#include <config.h>
+#include "config.h"
 #include "gtktextdisplay.h"
 #include "gtkintl.h"
-#include "gtkalias.h"
 /* DO NOT go putting private headers in here. This file should only
  * use the semi-public headers, as with gtktextview.c.
  */
@@ -93,28 +92,31 @@
 typedef struct _GtkTextRenderer      GtkTextRenderer;
 typedef struct _GtkTextRendererClass GtkTextRendererClass;
 
+enum {
+  NORMAL,
+  SELECTED,
+  CURSOR
+};
+
 struct _GtkTextRenderer
 {
-  GdkPangoRenderer parent_instance;
-
-  GdkScreen *screen;
+  PangoRenderer parent_instance;
 
   GtkWidget *widget;
-  GdkDrawable *drawable;
-  GdkRectangle clip_rect;
+  cairo_t *cr;
   
   GdkColor *error_color;	/* Error underline color for this widget */
   GList *widgets;		/* widgets encountered when drawing */
   
-  gboolean selected;
+  int state;
 };
 
 struct _GtkTextRendererClass
 {
-  GdkPangoRendererClass parent_class;
+  PangoRendererClass parent_class;
 };
 
-G_DEFINE_TYPE (GtkTextRenderer, _gtk_text_renderer, GDK_TYPE_PANGO_RENDERER)
+G_DEFINE_TYPE (GtkTextRenderer, _gtk_text_renderer, PANGO_TYPE_RENDERER)
 
 static GdkColor *
 text_renderer_get_error_color (GtkTextRenderer *text_renderer)
@@ -177,30 +179,30 @@ gtk_text_renderer_prepare_run (PangoRenderer  *renderer,
 			       PangoLayoutRun *run)
 {
   GtkTextRenderer *text_renderer = GTK_TEXT_RENDERER (renderer);
-  GdkPangoRenderer *gdk_renderer = GDK_PANGO_RENDERER (renderer);
   GdkColor *bg_color, *fg_color, *underline_color;
-  GdkPixmap *fg_stipple, *bg_stipple;
   GtkTextAppearance *appearance;
 
   PANGO_RENDERER_CLASS (_gtk_text_renderer_parent_class)->prepare_run (renderer, run);
 
   appearance = get_item_appearance (run->item);
   g_assert (appearance != NULL);
-  
-  if (appearance->draw_bg && !text_renderer->selected)
+
+  if (appearance->draw_bg && text_renderer->state == NORMAL)
     bg_color = &appearance->bg_color;
   else
     bg_color = NULL;
   
   text_renderer_set_gdk_color (text_renderer, PANGO_RENDER_PART_BACKGROUND, bg_color);
 
-  if (text_renderer->selected)
+  if (text_renderer->state == SELECTED)
     {
-      if (GTK_WIDGET_HAS_FOCUS (text_renderer->widget))
+      if (gtk_widget_has_focus (text_renderer->widget))
 	fg_color = &text_renderer->widget->style->text[GTK_STATE_SELECTED];
       else
 	fg_color = &text_renderer->widget->style->text[GTK_STATE_ACTIVE];
     }
+  else if (text_renderer->state == CURSOR && gtk_widget_has_focus (text_renderer->widget))
+    fg_color = &text_renderer->widget->style->base[GTK_STATE_NORMAL];
   else
     fg_color = &appearance->fg_color;
 
@@ -213,31 +215,134 @@ gtk_text_renderer_prepare_run (PangoRenderer  *renderer,
     underline_color = fg_color;
 
   text_renderer_set_gdk_color (text_renderer, PANGO_RENDER_PART_UNDERLINE, underline_color);
+}
 
-  fg_stipple = appearance->fg_stipple;
-  if (fg_stipple && text_renderer->screen != gdk_drawable_get_screen (fg_stipple))
-    {
-      g_warning ("gtk_text_renderer_prepare_run:\n"
-		 "The foreground stipple bitmap has been created on the wrong screen.\n"
-		 "Ignoring the stipple bitmap information.");
-      fg_stipple = NULL;
-    }
-      
-  gdk_pango_renderer_set_stipple (gdk_renderer, PANGO_RENDER_PART_FOREGROUND, fg_stipple);
-  gdk_pango_renderer_set_stipple (gdk_renderer, PANGO_RENDER_PART_STRIKETHROUGH, fg_stipple);
-  gdk_pango_renderer_set_stipple (gdk_renderer, PANGO_RENDER_PART_UNDERLINE, fg_stipple);
+static void
+set_color (GtkTextRenderer *text_renderer,
+           PangoRenderPart  part)
+{
+  PangoColor *color;
 
-  bg_stipple = appearance->draw_bg ? appearance->bg_stipple : NULL;
-  
-  if (bg_stipple && text_renderer->screen != gdk_drawable_get_screen (bg_stipple))
-    {
-      g_warning ("gtk_text_renderer_prepare_run:\n"
-		 "The background stipple bitmap has been created on the wrong screen.\n"
-		 "Ignoring the stipple bitmap information.");
-      bg_stipple = NULL;
-    }
-  
-  gdk_pango_renderer_set_stipple (gdk_renderer, PANGO_RENDER_PART_BACKGROUND, bg_stipple);
+  cairo_save (text_renderer->cr);
+
+  color = pango_renderer_get_color (PANGO_RENDERER (text_renderer), part);
+  if (color)
+    cairo_set_source_rgb (text_renderer->cr,
+                          color->red / 65535.,
+                          color->green / 65535.,
+                          color->blue / 65535.);
+}
+
+static void
+unset_color (GtkTextRenderer *text_renderer)
+{
+  cairo_restore (text_renderer->cr);
+}
+
+static void
+gtk_text_renderer_draw_glyphs (PangoRenderer     *renderer,
+                               PangoFont         *font,
+                               PangoGlyphString  *glyphs,
+                               int                x,
+                               int                y)
+{
+  GtkTextRenderer *text_renderer = GTK_TEXT_RENDERER (renderer);
+
+  set_color (text_renderer, PANGO_RENDER_PART_FOREGROUND);
+
+  cairo_move_to (text_renderer->cr, (double)x / PANGO_SCALE, (double)y / PANGO_SCALE);
+  pango_cairo_show_glyph_string (text_renderer->cr, font, glyphs);
+
+  unset_color (text_renderer);
+}
+
+static void
+gtk_text_renderer_draw_glyph_item (PangoRenderer     *renderer,
+                                   const char        *text,
+                                   PangoGlyphItem    *glyph_item,
+                                   int                x,
+                                   int                y)
+{
+  GtkTextRenderer *text_renderer = GTK_TEXT_RENDERER (renderer);
+
+  set_color (text_renderer, PANGO_RENDER_PART_FOREGROUND);
+
+  cairo_move_to (text_renderer->cr, (double)x / PANGO_SCALE, (double)y / PANGO_SCALE);
+  pango_cairo_show_glyph_item (text_renderer->cr, text, glyph_item);
+
+  unset_color (text_renderer);
+}
+
+static void
+gtk_text_renderer_draw_rectangle (PangoRenderer     *renderer,
+				  PangoRenderPart    part,
+				  int                x,
+				  int                y,
+				  int                width,
+				  int                height)
+{
+  GtkTextRenderer *text_renderer = GTK_TEXT_RENDERER (renderer);
+
+  set_color (text_renderer, part);
+
+  cairo_rectangle (text_renderer->cr,
+                   (double)x / PANGO_SCALE, (double)y / PANGO_SCALE,
+		   (double)width / PANGO_SCALE, (double)height / PANGO_SCALE);
+  cairo_fill (text_renderer->cr);
+
+  unset_color (text_renderer);
+}
+
+static void
+gtk_text_renderer_draw_trapezoid (PangoRenderer     *renderer,
+				  PangoRenderPart    part,
+				  double             y1_,
+				  double             x11,
+				  double             x21,
+				  double             y2,
+				  double             x12,
+				  double             x22)
+{
+  GtkTextRenderer *text_renderer = GTK_TEXT_RENDERER (renderer);
+  cairo_t *cr;
+  cairo_matrix_t matrix;
+
+  set_color (text_renderer, part);
+
+  cr = text_renderer->cr;
+
+  cairo_get_matrix (cr, &matrix);
+  matrix.xx = matrix.yy = 1.0;
+  matrix.xy = matrix.yx = 0.0;
+  cairo_set_matrix (cr, &matrix);
+
+  cairo_move_to (cr, x11, y1_);
+  cairo_line_to (cr, x21, y1_);
+  cairo_line_to (cr, x22, y2);
+  cairo_line_to (cr, x12, y2);
+  cairo_close_path (cr);
+
+  cairo_fill (cr);
+
+  unset_color (text_renderer);
+}
+
+static void
+gtk_text_renderer_draw_error_underline (PangoRenderer *renderer,
+					int            x,
+					int            y,
+					int            width,
+					int            height)
+{
+  GtkTextRenderer *text_renderer = GTK_TEXT_RENDERER (renderer);
+
+  set_color (text_renderer, PANGO_RENDER_PART_UNDERLINE);
+
+  pango_cairo_show_error_underline (text_renderer->cr,
+                                    (double)x / PANGO_SCALE, (double)y / PANGO_SCALE,
+                                    (double)width / PANGO_SCALE, (double)height / PANGO_SCALE);
+
+  unset_color (text_renderer);
 }
 
 static void
@@ -247,78 +352,69 @@ gtk_text_renderer_draw_shape (PangoRenderer   *renderer,
 			      int              y)
 {
   GtkTextRenderer *text_renderer = GTK_TEXT_RENDERER (renderer);
-  GdkGC *fg_gc;
+  GdkColor *fg;
 
-  if (text_renderer->selected)
+  if (text_renderer->state == SELECTED)
     {
-      if (GTK_WIDGET_HAS_FOCUS (text_renderer->widget))
-	fg_gc = text_renderer->widget->style->text_gc[GTK_STATE_SELECTED];
+      if (gtk_widget_has_focus (text_renderer->widget))
+	fg = &text_renderer->widget->style->text[GTK_STATE_SELECTED];
       else
-	fg_gc = text_renderer->widget->style->text_gc[GTK_STATE_SELECTED];
+	fg = &text_renderer->widget->style->text[GTK_STATE_SELECTED];
     }
+  else if (text_renderer->state == CURSOR && gtk_widget_has_focus (text_renderer->widget))
+    fg = &text_renderer->widget->style->base[GTK_STATE_NORMAL];
   else
-    fg_gc = text_renderer->widget->style->text_gc[GTK_STATE_NORMAL];
+    fg = &text_renderer->widget->style->text[GTK_STATE_NORMAL];
   
   if (attr->data == NULL)
     {
       /* This happens if we have an empty widget anchor. Draw
        * something empty-looking.
        */
-      GdkRectangle shape_rect, draw_rect;
+      GdkRectangle shape_rect;
+      cairo_t *cr;
       
       shape_rect.x = PANGO_PIXELS (x);
       shape_rect.y = PANGO_PIXELS (y + attr->logical_rect.y);
       shape_rect.width = PANGO_PIXELS (x + attr->logical_rect.width) - shape_rect.x;
       shape_rect.height = PANGO_PIXELS (y + attr->logical_rect.y + attr->logical_rect.height) - shape_rect.y;
       
-      if (gdk_rectangle_intersect (&shape_rect, &text_renderer->clip_rect,
-				   &draw_rect))
-	{
-	  gdk_draw_rectangle (text_renderer->drawable, fg_gc,
-			      FALSE, shape_rect.x, shape_rect.y,
-			      shape_rect.width, shape_rect.height);
-	  
-	  gdk_draw_line (text_renderer->drawable, fg_gc,
-			 shape_rect.x, shape_rect.y,
-			 shape_rect.x + shape_rect.width,
-			 shape_rect.y + shape_rect.height);
-	  
-	  gdk_draw_line (text_renderer->drawable, fg_gc,
-			 shape_rect.x + shape_rect.width, shape_rect.y,
-			 shape_rect.x,
-			 shape_rect.y + shape_rect.height);
-	}
+      cr = text_renderer->cr;
+
+      cairo_save (cr);
+
+      cairo_set_line_width (cr, 1.0);
+      gdk_cairo_set_source_color (cr, fg);
+
+      cairo_rectangle (cr,
+                       shape_rect.x + 0.5, shape_rect.y + 0.5,
+                       shape_rect.width - 1, shape_rect.height - 1);
+      cairo_move_to (cr, shape_rect.x + 0.5, shape_rect.y + 0.5);
+      cairo_line_to (cr, 
+                     shape_rect.x + shape_rect.width - 0.5,
+                     shape_rect.y + shape_rect.height - 0.5);
+      cairo_move_to (cr, shape_rect.x + 0.5,
+                     shape_rect.y + shape_rect.height - 0.5);
+      cairo_line_to (cr, shape_rect.x + shape_rect.width - 0.5,
+                     shape_rect.y + 0.5);
+
+      cairo_stroke (cr);
+      
+      cairo_restore (cr);
     }
   else if (GDK_IS_PIXBUF (attr->data))
     {
-      gint width, height;
-      GdkRectangle pixbuf_rect, draw_rect;
-      GdkPixbuf *pixbuf;
+      cairo_t *cr = text_renderer->cr;
+      GdkPixbuf *pixbuf = GDK_PIXBUF (attr->data);
       
-      pixbuf = GDK_PIXBUF (attr->data);
-      
-      width = gdk_pixbuf_get_width (pixbuf);
-      height = gdk_pixbuf_get_height (pixbuf);
-      
-      pixbuf_rect.x = PANGO_PIXELS (x);
-      pixbuf_rect.y = PANGO_PIXELS (y) - height;
-      pixbuf_rect.width = width;
-      pixbuf_rect.height = height;
-      
-      if (gdk_rectangle_intersect (&pixbuf_rect, &text_renderer->clip_rect,
-				   &draw_rect))
-	{
-	  gdk_draw_pixbuf (text_renderer->drawable,
-			   fg_gc,
-			   pixbuf,
-			   draw_rect.x - pixbuf_rect.x,
-			   draw_rect.y - pixbuf_rect.y,
-			   draw_rect.x, draw_rect.y,
-			   draw_rect.width,
-			   draw_rect.height,
-			   GDK_RGB_DITHER_NORMAL,
-			   0, 0);
-	}
+      cairo_save (cr);
+
+      gdk_cairo_set_source_pixbuf (cr, pixbuf,
+                                   PANGO_PIXELS (x),
+                                   PANGO_PIXELS (y) -  gdk_pixbuf_get_height (pixbuf));
+      cairo_paint (cr);
+
+      cairo_restore (cr);
     }
   else if (GTK_IS_WIDGET (attr->data))
     {
@@ -352,31 +448,30 @@ _gtk_text_renderer_class_init (GtkTextRendererClass *klass)
   PangoRendererClass *renderer_class = PANGO_RENDERER_CLASS (klass);
   
   renderer_class->prepare_run = gtk_text_renderer_prepare_run;
+  renderer_class->draw_glyphs = gtk_text_renderer_draw_glyphs;
+  renderer_class->draw_glyph_item = gtk_text_renderer_draw_glyph_item;
+  renderer_class->draw_rectangle = gtk_text_renderer_draw_rectangle;
+  renderer_class->draw_trapezoid = gtk_text_renderer_draw_trapezoid;
+  renderer_class->draw_error_underline = gtk_text_renderer_draw_error_underline;
   renderer_class->draw_shape = gtk_text_renderer_draw_shape;
 
   object_class->finalize = gtk_text_renderer_finalize;
 }
 
 static void
-text_renderer_set_selected (GtkTextRenderer *text_renderer,
-			    gboolean         selected)
+text_renderer_set_state (GtkTextRenderer *text_renderer,
+			 int              state)
 {
-  text_renderer->selected = selected;
+  text_renderer->state = state;
 }
 
 static void
 text_renderer_begin (GtkTextRenderer *text_renderer,
-		     GtkWidget       *widget,
-		     GdkDrawable     *drawable,
-		     GdkRectangle    *clip_rect)
+                     GtkWidget       *widget,
+                     cairo_t         *cr)
 {
   text_renderer->widget = widget;
-  text_renderer->drawable = drawable;
-  text_renderer->clip_rect = *clip_rect;
-
-  gdk_pango_renderer_set_drawable (GDK_PANGO_RENDERER (text_renderer), drawable);
-  gdk_pango_renderer_set_gc (GDK_PANGO_RENDERER (text_renderer),
-			     widget->style->text_gc[widget->state]);
+  text_renderer->cr = cr;
 }
 
 /* Returns a GSList of (referenced) widgets encountered while drawing.
@@ -387,7 +482,7 @@ text_renderer_end (GtkTextRenderer *text_renderer)
   GList *widgets = text_renderer->widgets;
 
   text_renderer->widget = NULL;
-  text_renderer->drawable = NULL;
+  text_renderer->cr = NULL;
 
   text_renderer->widgets = NULL;
 
@@ -397,14 +492,11 @@ text_renderer_end (GtkTextRenderer *text_renderer)
       text_renderer->error_color = NULL;
     }
 
-  gdk_pango_renderer_set_drawable (GDK_PANGO_RENDERER (text_renderer), NULL);
-  gdk_pango_renderer_set_gc (GDK_PANGO_RENDERER (text_renderer), NULL);
-
   return widgets;
 }
 
 
-static GdkRegion *
+static cairo_region_t *
 get_selected_clip (GtkTextRenderer    *text_renderer,
                    PangoLayout        *layout,
                    PangoLayoutLine    *line,
@@ -416,8 +508,7 @@ get_selected_clip (GtkTextRenderer    *text_renderer,
 {
   gint *ranges;
   gint n_ranges, i;
-  GdkRegion *clip_region = gdk_region_new ();
-  GdkRegion *tmp_region;
+  cairo_region_t *clip_region = cairo_region_create ();
 
   pango_layout_line_get_x_ranges (line, start_index, end_index, &ranges, &n_ranges);
 
@@ -430,13 +521,9 @@ get_selected_clip (GtkTextRenderer    *text_renderer,
       rect.width = PANGO_PIXELS (ranges[2*i + 1]) - PANGO_PIXELS (ranges[2*i]);
       rect.height = height;
       
-      gdk_region_union_with_rect (clip_region, &rect);
+      cairo_region_union_rectangle (clip_region, &rect);
     }
 
-  tmp_region = gdk_region_rectangle (&text_renderer->clip_rect);
-  gdk_region_intersect (clip_region, tmp_region);
-  gdk_region_destroy (tmp_region);
-
   g_free (ranges);
   return clip_region;
 }
@@ -455,7 +542,7 @@ render_para (GtkTextRenderer    *text_renderer,
   PangoLayoutIter *iter;
   PangoRectangle layout_logical;
   int screen_width;
-  GdkGC *selection_gc, *fg_gc;
+  GdkColor *selection;
   gint state;
   
   gboolean first = TRUE;
@@ -471,21 +558,21 @@ render_para (GtkTextRenderer    *text_renderer,
 
   screen_width = line_display->total_width;
   
-  if (GTK_WIDGET_HAS_FOCUS (text_renderer->widget))
+  if (gtk_widget_has_focus (text_renderer->widget))
     state = GTK_STATE_SELECTED;
   else
     state = GTK_STATE_ACTIVE;
 
-  selection_gc = text_renderer->widget->style->base_gc [state];
-  fg_gc = text_renderer->widget->style->text_gc[text_renderer->widget->state];
+  selection = &text_renderer->widget->style->base [state];
 
   do
     {
-      PangoLayoutLine *line = pango_layout_iter_get_line (iter);
+      PangoLayoutLine *line = pango_layout_iter_get_line_readonly (iter);
       int selection_y, selection_height;
       int first_y, last_y;
       PangoRectangle line_rect;
       int baseline;
+      gboolean at_last_line;
       
       pango_layout_iter_get_line_extents (iter, NULL, &line_rect);
       baseline = pango_layout_iter_get_baseline (iter);
@@ -508,8 +595,9 @@ render_para (GtkTextRenderer    *text_renderer,
           selection_y -= line_display->top_margin;
           selection_height += line_display->top_margin;
         }
-      
-      if (pango_layout_iter_at_last_line (iter))
+
+      at_last_line = pango_layout_iter_at_last_line (iter);
+      if (at_last_line)
         selection_height += line_display->bottom_margin;
       
       first = FALSE;
@@ -517,15 +605,17 @@ render_para (GtkTextRenderer    *text_renderer,
       if (selection_start_index < byte_offset &&
           selection_end_index > line->length + byte_offset) /* All selected */
         {
-          gdk_draw_rectangle (text_renderer->drawable,
-                              selection_gc,
-                              TRUE,
-                              x + line_display->left_margin,
-                              selection_y,
-                              screen_width,
-                              selection_height);
+          cairo_t *cr = text_renderer->cr;
 
-	  text_renderer_set_selected (text_renderer, TRUE);
+          cairo_save (cr);
+          gdk_cairo_set_source_color (cr, selection);
+          cairo_rectangle (cr, 
+                           x + line_display->left_margin, selection_y,
+                           screen_width, selection_height);
+          cairo_fill (cr);
+          cairo_restore(cr);
+
+	  text_renderer_set_state (text_renderer, SELECTED);
 	  pango_renderer_draw_layout_line (PANGO_RENDERER (text_renderer),
 					   line, 
 					   PANGO_SCALE * x + line_rect.x,
@@ -535,24 +625,20 @@ render_para (GtkTextRenderer    *text_renderer,
         {
           if (line_display->pg_bg_color)
             {
-              GdkGC *bg_gc;
-              
-              bg_gc = gdk_gc_new (text_renderer->drawable);
-              gdk_gc_set_fill (bg_gc, GDK_SOLID);
-              gdk_gc_set_rgb_fg_color (bg_gc, line_display->pg_bg_color);
-            
-              gdk_draw_rectangle (text_renderer->drawable,
-                                  bg_gc,
-                                  TRUE,
-                                  x + line_display->left_margin,
-                                  selection_y,
-                                  screen_width,
-                                  selection_height);
-              
-              g_object_unref (bg_gc);
+              cairo_t *cr = text_renderer->cr;
+
+              cairo_save (cr);
+
+              gdk_cairo_set_source_color (cr, line_display->pg_bg_color);
+              cairo_rectangle (cr, 
+                               x + line_display->left_margin, selection_y,
+                               screen_width, selection_height);
+              cairo_fill (cr);
+
+              cairo_restore (cr);
             }
         
-	  text_renderer_set_selected (text_renderer, FALSE);
+	  text_renderer_set_state (text_renderer, NORMAL);
 	  pango_renderer_draw_layout_line (PANGO_RENDERER (text_renderer),
 					   line, 
 					   PANGO_SCALE * x + line_rect.x,
@@ -566,54 +652,50 @@ render_para (GtkTextRenderer    *text_renderer,
 	       (selection_start_index == byte_offset + line->length && pango_layout_iter_at_last_line (iter))) &&
 	      selection_end_index > byte_offset)
             {
-              GdkRegion *clip_region = get_selected_clip (text_renderer, layout, line,
+              cairo_t *cr = text_renderer->cr;
+              cairo_region_t *clip_region = get_selected_clip (text_renderer, layout, line,
                                                           x + line_display->x_offset,
                                                           selection_y,
                                                           selection_height,
                                                           selection_start_index, selection_end_index);
 
-	      /* When we change the clip on the foreground GC, we have to set
-	       * it on the rendererer again, since the rendererer might have
-	       * copied the GC to change attributes.
-	       */
-	      gdk_pango_renderer_set_gc (GDK_PANGO_RENDERER (text_renderer), NULL);
-              gdk_gc_set_clip_region (selection_gc, clip_region);
-              gdk_gc_set_clip_region (fg_gc, clip_region);
-	      gdk_pango_renderer_set_gc (GDK_PANGO_RENDERER (text_renderer), fg_gc);
+              cairo_save (cr);
+              gdk_cairo_region (cr, clip_region);
+              cairo_clip (cr);
+              cairo_region_destroy (clip_region);
 
-              gdk_draw_rectangle (text_renderer->drawable,
-                                  selection_gc,
-                                  TRUE,
-                                  x + PANGO_PIXELS (line_rect.x),
-                                  selection_y,
-                                  PANGO_PIXELS (line_rect.width),
-                                  selection_height);
+              gdk_cairo_set_source_color (cr, selection);
+              cairo_rectangle (cr,
+                               x + PANGO_PIXELS (line_rect.x),
+                               selection_y,
+                               PANGO_PIXELS (line_rect.width),
+                               selection_height);
+              cairo_fill (cr);
 
-	      text_renderer_set_selected (text_renderer, TRUE);
+	      text_renderer_set_state (text_renderer, SELECTED);
 	      pango_renderer_draw_layout_line (PANGO_RENDERER (text_renderer),
 					       line, 
 					       PANGO_SCALE * x + line_rect.x,
 					       PANGO_SCALE * y + baseline);
 
-	      gdk_pango_renderer_set_gc (GDK_PANGO_RENDERER (text_renderer), NULL);
-              gdk_gc_set_clip_region (selection_gc, NULL);
-              gdk_gc_set_clip_region (fg_gc, NULL);
-	      gdk_pango_renderer_set_gc (GDK_PANGO_RENDERER (text_renderer), fg_gc);
-	      
-              gdk_region_destroy (clip_region);
+              cairo_restore (cr);
 
               /* Paint in the ends of the line */
               if (line_rect.x > line_display->left_margin * PANGO_SCALE &&
                   ((line_display->direction == GTK_TEXT_DIR_LTR && selection_start_index < byte_offset) ||
                    (line_display->direction == GTK_TEXT_DIR_RTL && selection_end_index > byte_offset + line->length)))
                 {
-                  gdk_draw_rectangle (text_renderer->drawable,
-                                      selection_gc,
-                                      TRUE,
-                                      x + line_display->left_margin,
-                                      selection_y,
-                                      PANGO_PIXELS (line_rect.x) - line_display->left_margin,
-                                      selection_height);
+                  cairo_save (cr);
+
+                  gdk_cairo_set_source_color (cr, selection);
+                  cairo_rectangle (cr,
+                                   x + line_display->left_margin,
+                                   selection_y,
+                                   PANGO_PIXELS (line_rect.x) - line_display->left_margin,
+                                   selection_height);
+                  cairo_fill (cr);
+
+                  cairo_restore (cr);
                 }
 
               if (line_rect.x + line_rect.width <
@@ -627,15 +709,63 @@ render_para (GtkTextRenderer    *text_renderer,
                     line_display->left_margin + screen_width -
                     PANGO_PIXELS (line_rect.x) - PANGO_PIXELS (line_rect.width);
 
-                  gdk_draw_rectangle (text_renderer->drawable,
-                                      selection_gc,
-                                      TRUE,
-                                      x + PANGO_PIXELS (line_rect.x) + PANGO_PIXELS (line_rect.width),
-                                      selection_y,
-                                      nonlayout_width,
-                                      selection_height);
+                  cairo_save (cr);
+
+                  gdk_cairo_set_source_color (cr, selection);
+                  cairo_rectangle (cr,
+                                   x + PANGO_PIXELS (line_rect.x) + PANGO_PIXELS (line_rect.width),
+                                   selection_y,
+                                   nonlayout_width,
+                                   selection_height);
+                  cairo_fill (cr);
+
+                  cairo_restore (cr);
                 }
             }
+	  else if (line_display->has_block_cursor &&
+		   gtk_widget_has_focus (text_renderer->widget) &&
+		   byte_offset <= line_display->insert_index &&
+		   (line_display->insert_index < byte_offset + line->length ||
+		    (at_last_line && line_display->insert_index == byte_offset + line->length)))
+	    {
+	      GdkRectangle cursor_rect;
+              GdkColor cursor_color;
+              cairo_t *cr = text_renderer->cr;
+
+	      /* we draw text using base color on filled cursor rectangle of cursor color
+	       * (normally white on black) */
+	      _gtk_widget_get_cursor_color (text_renderer->widget, &cursor_color);
+
+	      cursor_rect.x = x + line_display->x_offset + line_display->block_cursor.x;
+	      cursor_rect.y = y + line_display->block_cursor.y + line_display->top_margin;
+	      cursor_rect.width = line_display->block_cursor.width;
+	      cursor_rect.height = line_display->block_cursor.height;
+
+              cairo_save (cr);
+
+              gdk_cairo_rectangle (cr, &cursor_rect);
+              cairo_clip (cr);
+
+              gdk_cairo_set_source_color (cr, &cursor_color);
+              cairo_paint (cr);
+
+	      /* draw text under the cursor if any */
+	      if (!line_display->cursor_at_line_end)
+		{
+                  gdk_cairo_set_source_color (cr,
+                                              &text_renderer->widget->style->base[text_renderer->widget->state]);
+
+		  text_renderer_set_state (text_renderer, CURSOR);
+
+		  pango_renderer_draw_layout_line (PANGO_RENDERER (text_renderer),
+						   line,
+						   PANGO_SCALE * x + line_rect.x,
+						   PANGO_SCALE * y + baseline);
+
+		}
+
+              cairo_restore (cr);
+	    }
         }
 
       byte_offset += line->length;
@@ -645,37 +775,13 @@ render_para (GtkTextRenderer    *text_renderer,
   pango_layout_iter_free (iter);
 }
 
-static void
-on_renderer_display_closed (GdkDisplay       *display,
-                            gboolean          is_error,
-			    GtkTextRenderer  *text_renderer)
-{
-  g_signal_handlers_disconnect_by_func (text_renderer->screen,
-					(gpointer)on_renderer_display_closed,
-					text_renderer);
-  g_object_set_data (G_OBJECT (text_renderer->screen), I_("gtk-text-renderer"), NULL);
-}
-
 static GtkTextRenderer *
-get_text_renderer (GdkScreen *screen)
+get_text_renderer (void)
 {
-  GtkTextRenderer *text_renderer;
+  static GtkTextRenderer *text_renderer = NULL;
 
-  g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
-  
-  text_renderer = g_object_get_data (G_OBJECT (screen), "gtk-text-renderer");
   if (!text_renderer)
-    {
-      text_renderer = g_object_new (GTK_TYPE_TEXT_RENDERER, "screen", screen, NULL);
-      text_renderer->screen = screen;
-      
-      g_object_set_data_full (G_OBJECT (screen), I_("gtk-text-renderer"), text_renderer,
-			      (GDestroyNotify)g_object_unref);
-
-      g_signal_connect_object (gdk_screen_get_display (screen), "closed",
-                               G_CALLBACK (on_renderer_display_closed),
-                               text_renderer, 0);
-    }
+    text_renderer = g_object_new (GTK_TYPE_TEXT_RENDERER, NULL);
 
   return text_renderer;
 }
@@ -684,7 +790,7 @@ void
 gtk_text_layout_draw (GtkTextLayout *layout,
                       GtkWidget *widget,
                       GdkDrawable *drawable,
-		      GdkGC       *cursor_gc,
+		      gpointer     cursor_gc,
                       /* Location of the drawable
                          in layout coordinates */
                       gint x_offset,
@@ -707,6 +813,7 @@ gtk_text_layout_draw (GtkTextLayout *layout,
   GSList *line_list;
   GSList *tmp_list;
   GList *tmp_widgets;
+  cairo_t *cr;
   
   g_return_if_fail (GTK_IS_TEXT_LAYOUT (layout));
   g_return_if_fail (layout->default_style != NULL);
@@ -724,14 +831,14 @@ gtk_text_layout_draw (GtkTextLayout *layout,
   if (line_list == NULL)
     return; /* nothing on the screen */
 
-  clip.x = x;
-  clip.y = y;
-  clip.width = width;
-  clip.height = height;
+  cr = gdk_cairo_create (drawable);
+  cairo_rectangle (cr, x, y, width, height);
+  cairo_clip (cr);
 
-  text_renderer = get_text_renderer (gdk_drawable_get_screen (drawable));
+  gdk_cairo_set_source_color (cr, &widget->style->text[widget->state]);
 
-  text_renderer_begin (text_renderer, widget, drawable, &clip);
+  text_renderer = get_text_renderer ();
+  text_renderer_begin (text_renderer, widget, cr);
 
   gtk_text_layout_wrap_loop_start (layout);
 
@@ -852,8 +959,7 @@ gtk_text_layout_draw (GtkTextLayout *layout,
       g_list_free (tmp_widgets);
     }
 
+  cairo_destroy (cr);
+
   g_slist_free (line_list);
 }
-
-#define __GTK_TEXT_DISPLAY_C__
-#include "gtkaliasdef.c"
