@@ -1,7 +1,6 @@
 *tp++ = gdk_app_launch_context_get_type();
 *tp++ = gdk_axis_use_get_type();
 *tp++ = gdk_byte_order_get_type();
-*tp++ = gdk_cap_style_get_type();
 *tp++ = gdk_color_get_type();
 *tp++ = gdk_colormap_get_type();
 *tp++ = gdk_colorspace_get_type();
@@ -21,34 +20,22 @@
 *tp++ = gdk_event_mask_get_type();
 *tp++ = gdk_event_type_get_type();
 *tp++ = gdk_extension_mode_get_type();
-*tp++ = gdk_fill_get_type();
-*tp++ = gdk_fill_rule_get_type();
 *tp++ = gdk_filter_return_get_type();
-*tp++ = gdk_font_get_type();
-*tp++ = gdk_font_type_get_type();
-*tp++ = gdk_function_get_type();
-*tp++ = gdk_gc_get_type();
-*tp++ = gdk_gc_values_mask_get_type();
 *tp++ = gdk_grab_ownership_get_type();
 *tp++ = gdk_grab_status_get_type();
 *tp++ = gdk_gravity_get_type();
-*tp++ = gdk_image_get_type();
-*tp++ = gdk_image_type_get_type();
 *tp++ = gdk_input_mode_get_type();
 *tp++ = gdk_input_source_get_type();
 *tp++ = gdk_interp_type_get_type();
-*tp++ = gdk_join_style_get_type();
 *tp++ = gdk_keymap_get_type();
-*tp++ = gdk_line_style_get_type();
 *tp++ = gdk_modifier_type_get_type();
 *tp++ = gdk_notify_type_get_type();
-*tp++ = gdk_overlap_type_get_type();
 *tp++ = gdk_owner_change_get_type();
-*tp++ = gdk_pango_renderer_get_type();
 *tp++ = gdk_pixbuf_alpha_mode_get_type();
 *tp++ = gdk_pixbuf_animation_get_type();
 *tp++ = gdk_pixbuf_animation_iter_get_type();
 *tp++ = gdk_pixbuf_error_get_type();
+*tp++ = gdk_pixbuf_format_get_type();
 *tp++ = gdk_pixbuf_get_type();
 *tp++ = gdk_pixbuf_loader_get_type();
 *tp++ = gdk_pixbuf_rotation_get_type();
@@ -58,12 +45,10 @@
 *tp++ = gdk_property_state_get_type();
 *tp++ = gdk_prop_mode_get_type();
 *tp++ = gdk_rectangle_get_type();
-*tp++ = gdk_rgb_dither_get_type();
 *tp++ = gdk_screen_get_type();
 *tp++ = gdk_scroll_direction_get_type();
 *tp++ = gdk_setting_action_get_type();
 *tp++ = gdk_status_get_type();
-*tp++ = gdk_subwindow_mode_get_type();
 *tp++ = gdk_visibility_state_get_type();
 *tp++ = gdk_visual_get_type();
 *tp++ = gdk_visual_type_get_type();
@@ -198,6 +183,7 @@
 *tp++ = gtk_justification_get_type();
 *tp++ = gtk_label_get_type();
 *tp++ = gtk_layout_get_type();
+*tp++ = gtk_license_get_type();
 *tp++ = gtk_link_button_get_type();
 *tp++ = gtk_list_store_get_type();
 *tp++ = gtk_menu_bar_get_type();
