@@ -205,7 +205,7 @@ gail_item_get_name (AtkObject *obj)
       GtkWidget *widget;
       GtkWidget *label;
 
-      widget = GTK_ACCESSIBLE (obj)->widget;
+      widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (obj));
       if (widget == NULL)
         /*
          * State is defunct
@@ -244,7 +244,7 @@ gail_item_get_name (AtkObject *obj)
               parent_obj = atk_object_get_parent (gtk_widget_get_accessible (parent));
               if (GTK_IS_ACCESSIBLE (parent_obj))
                 {
-                  parent = GTK_ACCESSIBLE (parent_obj)->widget;
+                  parent = gtk_accessible_get_widget (GTK_ACCESSIBLE (parent_obj));
                   if (GTK_IS_COMBO_BOX (parent))
                     {
                       GtkTreeModel *model;
@@ -360,7 +360,7 @@ gail_item_get_text (AtkText *text,
   GailItem *item;
   const gchar *label_text;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return NULL;
@@ -396,7 +396,7 @@ gail_item_get_text_before_offset (AtkText         *text,
   GtkWidget *label;
   GailItem *item;
   
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   
   if (widget == NULL)
     /* State is defunct */
@@ -428,7 +428,7 @@ gail_item_get_text_at_offset (AtkText         *text,
   GtkWidget *label;
   GailItem *item;
  
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   
   if (widget == NULL)
     /* State is defunct */
@@ -460,7 +460,7 @@ gail_item_get_text_after_offset (AtkText         *text,
   GtkWidget *label;
   GailItem *item;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   
   if (widget == NULL)
   {
@@ -489,7 +489,7 @@ gail_item_get_character_count (AtkText *text)
   GtkWidget *widget;
   GtkWidget *label;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return 0;
@@ -517,7 +517,7 @@ gail_item_get_character_extents (AtkText      *text,
   gint index, x_layout, y_layout;
   const gchar *label_text;
  
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
 
   if (widget == NULL)
     /* State is defunct */
@@ -548,7 +548,7 @@ gail_item_get_offset_at_point (AtkText      *text,
   gint index, x_layout, y_layout;
   const gchar *label_text;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return -1;
@@ -587,7 +587,7 @@ gail_item_get_run_attributes (AtkText *text,
   GtkJustification justify;
   GtkTextDirection dir;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return NULL;
@@ -629,7 +629,7 @@ gail_item_get_default_attributes (AtkText *text)
   GtkWidget *label;
   AtkAttributeSet *at_set = NULL;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return NULL;
@@ -654,7 +654,7 @@ gail_item_get_character_at_offset (AtkText *text,
   const gchar *string;
   gchar *index;
 
-  widget = GTK_ACCESSIBLE (text)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
     /* State is defunct */
     return '\0';
