@@ -21,8 +21,6 @@
 
 #include <string.h>
 
-#undef GTK_DISABLE_DEPRECATED
-
 #include <gtk/gtk.h>
 
 #include "gailwindow.h"
@@ -210,7 +208,7 @@ gail_window_real_initialize (AtkObject *obj,
   g_object_set_data (G_OBJECT (obj), "atk-component-layer",
                      GINT_TO_POINTER (ATK_LAYER_WINDOW));
 
-  if (GTK_IS_FILE_SELECTION (widget))
+  if (GTK_IS_FILE_CHOOSER_DIALOG (widget))
     obj->role = ATK_ROLE_FILE_CHOOSER;
   else if (GTK_IS_COLOR_SELECTION_DIALOG (widget))
     obj->role = ATK_ROLE_COLOR_CHOOSER;
@@ -225,8 +223,8 @@ gail_window_real_initialize (AtkObject *obj,
       const gchar *name;
 
       name = gtk_widget_get_name (widget);
-      if (name && (!strcmp (name, "gtk-tooltip") ||
-                   !strcmp (name, "gtk-tooltips")))
+
+      if (!g_strcmp0 (name, "gtk-tooltip"))
         obj->role = ATK_ROLE_TOOL_TIP;
       else if (GTK_IS_PLUG (widget))
         obj->role = ATK_ROLE_PANEL;
@@ -274,7 +272,7 @@ gail_window_get_name (AtkObject *accessible)
       /*
        * Get the window title if it exists
        */
-      GtkWidget* widget = GTK_ACCESSIBLE (accessible)->widget; 
+      GtkWidget* widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (accessible));
 
       if (widget == NULL)
         /*
@@ -340,7 +338,7 @@ gail_window_get_parent (AtkObject *accessible)
 static gint
 gail_window_get_index_in_parent (AtkObject *accessible)
 {
-  GtkWidget* widget = GTK_ACCESSIBLE (accessible)->widget; 
+  GtkWidget* widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (accessible));
   AtkObject* atk_obj = atk_get_root ();
   gint index = -1;
 
@@ -401,7 +399,7 @@ gail_window_ref_relation_set (AtkObject *obj)
 
   gail_return_val_if_fail (GAIL_IS_WIDGET (obj), NULL);
 
-  widget = GTK_ACCESSIBLE (obj)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (obj));
   if (widget == NULL)
     /*
      * State is defunct
@@ -418,7 +416,7 @@ gail_window_ref_relation_set (AtkObject *obj)
         {
           atk_relation_set_remove (relation_set, relation);
         }
-      if (gtk_widget_get_visible(widget) && gtk_tooltips_get_info_from_tip_window (GTK_WINDOW (widget), NULL, &current_widget))
+      if (gtk_widget_get_visible(widget) && FALSE /* FIXME gtk_tooltips_get_info_from_tip_window (GTK_WINDOW (widget), NULL, &current_widget) */)
         {
           array [0] = gtk_widget_get_accessible (current_widget);
 
@@ -439,7 +437,7 @@ gail_window_ref_state_set (AtkObject *accessible)
   GdkWindowState state;
 
   state_set = ATK_OBJECT_CLASS (gail_window_parent_class)->ref_state_set (accessible);
-  widget = GTK_ACCESSIBLE (accessible)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (accessible));
  
   if (widget == NULL)
     return state_set;
@@ -472,7 +470,7 @@ idle_notify_name_change (gpointer data)
 
   window = GAIL_WINDOW (data);
   window->name_change_handler = 0;
-  if (GTK_ACCESSIBLE (window)->widget == NULL)
+  if (gtk_accessible_get_widget (GTK_ACCESSIBLE (window)) == NULL)
     return FALSE;
 
   obj = ATK_OBJECT (window);
@@ -551,7 +549,7 @@ gail_window_get_extents (AtkComponent  *component,
                          gint          *height,
                          AtkCoordType  coord_type)
 {
-  GtkWidget *widget = GTK_ACCESSIBLE (component)->widget; 
+  GtkWidget *widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (component)); 
   GdkRectangle rect;
   gint x_toplevel, y_toplevel;
 
@@ -597,7 +595,7 @@ gail_window_get_size (AtkComponent *component,
                       gint         *width,
                       gint         *height)
 {
-  GtkWidget *widget = GTK_ACCESSIBLE (component)->widget; 
+  GtkWidget *widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (component));
   GdkRectangle rect;
 
   if (widget == NULL)
@@ -662,10 +660,10 @@ get_window_desktop (Window window)
 
   if (_net_wm_desktop == None)
     _net_wm_desktop =
-		XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), "_NET_WM_DESKTOP", False);
+		XInternAtom (gdk_display, "_NET_WM_DESKTOP", False);
 
   gdk_error_trap_push ();
-  result = XGetWindowProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), window, _net_wm_desktop,
+  result = XGetWindowProperty (gdk_display, window, _net_wm_desktop,
                                0, G_MAXLONG,
                                False, XA_CARDINAL,
                                &ret_type, &format, &nitems,
@@ -716,11 +714,11 @@ get_stacked_windows (GailScreenInfo *info)
 
   if (_net_client_list_stacking == None)
     _net_client_list_stacking =
-		XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), "_NET_CLIENT_LIST_STACKING", False);
+		XInternAtom (gdk_display, "_NET_CLIENT_LIST_STACKING", False);
 
   gdk_error_trap_push ();
   ret_type = None;
-  result = XGetWindowProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+  result = XGetWindowProperty (gdk_display,
                                GDK_WINDOW_XWINDOW (info->root_window),
                                _net_client_list_stacking,
                                0, G_MAXLONG,
@@ -917,11 +915,11 @@ init_gail_screen (GdkScreen *screen,
 
   get_stacked_windows (&gail_screens [screen_n]);
 
-  XGetWindowAttributes (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+  XGetWindowAttributes (gdk_display,
 			GDK_WINDOW_XWINDOW (gail_screens [screen_n].root_window),
 			&attrs); 
 
-  XSelectInput (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+  XSelectInput (gdk_display,
 		GDK_WINDOW_XWINDOW (gail_screens [screen_n].root_window),
 		attrs.your_event_mask | PropertyChangeMask);
            
@@ -1002,7 +1000,7 @@ get_window_zorder (GdkWindow *window)
 static gint
 gail_window_get_mdi_zorder (AtkComponent *component)
 {
-  GtkWidget *widget = GTK_ACCESSIBLE (component)->widget;
+  GtkWidget *widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (component));
 
   if (widget == NULL)
     /*
@@ -1020,7 +1018,7 @@ gail_window_get_mdi_zorder (AtkComponent *component)
 static gint
 gail_window_get_mdi_zorder (AtkComponent *component)
 {
-  GtkWidget *widget = GTK_ACCESSIBLE (component)->widget;
+  GtkWidget *widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (component));
 
   if (widget == NULL)
     /*
@@ -1038,7 +1036,7 @@ gail_window_get_mdi_zorder (AtkComponent *component)
 static gint
 gail_window_get_mdi_zorder (AtkComponent *component)
 {
-  GtkWidget *widget = GTK_ACCESSIBLE (component)->widget;
+  GtkWidget *widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (component));
 
   if (widget == NULL)
     /*
