@@ -1,6 +1,4 @@
 #define MAX_BUFFER 256
-#undef GTK_DISABLE_DEPRECATED
-#define GTK_ENABLE_BROKEN
 #define MAX_GROUPS 20
 #define MAX_NAME_VALUE 20
 
@@ -576,10 +574,11 @@ _object_is_ours (AtkObject *aobject)
    */
    if (aobject != NULL)
      {
-       if (GTK_ACCESSIBLE(toplevel)->widget == mainWindow)
-         {
+       GtkWidget *widget;
+
+       widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (toplevel));
+       if (widget == mainWindow)
            return TRUE;
-         }
      }
 
   return FALSE;
@@ -617,6 +616,7 @@ ferret_get_name_from_container (AtkObject *aobject)
 static gint
 _print_object (AtkObject *aobject)
 {
+    GtkWidget *widget;
     G_CONST_RETURN gchar * parent_name = NULL;
     G_CONST_RETURN gchar * name = NULL;
     G_CONST_RETURN gchar * description = NULL;
@@ -656,11 +656,11 @@ _print_object (AtkObject *aobject)
         accel_name = "";
       }
 
-    if (GTK_IS_ACCESSIBLE (aobject) &&
-        GTK_IS_WIDGET (GTK_ACCESSIBLE (aobject)->widget))
+    widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (aobject));
+    if (widget)
       {
         _print_key_value(tab_n, group_num, "Widget name",
-          (gpointer)gtk_widget_get_name(GTK_ACCESSIBLE (aobject)->widget),
+          (gpointer)gtk_widget_get_name (widget),
           VALUE_STRING);
       }
     else
@@ -1666,7 +1666,6 @@ _create_window (void)
     {
         window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
         gtk_widget_set_name (window, "Ferret Window");
-        gtk_window_set_policy (GTK_WINDOW(window), TRUE, TRUE, FALSE);
 
         g_signal_connect (GTK_OBJECT (window), "destroy",
                            G_CALLBACK (gtk_widget_destroyed),
@@ -1684,7 +1683,7 @@ _create_window (void)
         gtk_box_pack_start (GTK_BOX (vbox1), menubar, FALSE, TRUE, 0);
         gtk_widget_show (menubar);
         menutop = gtk_menu_item_new_with_label("Menu");
-        gtk_menu_bar_append(GTK_MENU_BAR(menubar), menutop);
+        gtk_menu_shell_append (GTK_MENU_SHELL (menubar), menutop);
         gtk_widget_show (menutop);
         menu = gtk_menu_new();
         gtk_menu_item_set_submenu (GTK_MENU_ITEM (menutop), menu);
@@ -1978,7 +1977,7 @@ _get_name_value(GroupInfo *group, const gchar *label,
         name_value->label = GTK_LABEL(gtk_label_new(label));
         name_value->string = gtk_label_new (NULL);
         name_value->boolean = gtk_check_button_new ();
-        name_value->text = gtk_entry_new_with_max_length (1000);
+        gtk_entry_buffer_set_max_length (gtk_entry_get_buffer (GTK_ENTRY (name_value->text)), 1000);
         name_value->button = GTK_BUTTON(gtk_button_new ());
 
         gtk_box_pack_end(GTK_BOX(name_value->column1),
@@ -2611,7 +2610,7 @@ void _action_cb(GtkWidget *widget, gpointer  *userdata)
 void _toggle_terminal(GtkCheckMenuItem *checkmenuitem,
   gpointer user_data)
 {
-   if (checkmenuitem->active)
+   if (gtk_check_menu_item_get_active (checkmenuitem))
        display_ascii = TRUE;
    else
        display_ascii = FALSE;
@@ -2620,7 +2619,7 @@ void _toggle_terminal(GtkCheckMenuItem *checkmenuitem,
 void _toggle_no_signals(GtkCheckMenuItem *checkmenuitem,
   gpointer user_data)
 {
-   if (checkmenuitem->active)
+   if (gtk_check_menu_item_get_active (checkmenuitem))
        no_signals = TRUE;
    else
        no_signals = FALSE;
@@ -2629,7 +2628,7 @@ void _toggle_no_signals(GtkCheckMenuItem *checkmenuitem,
 void _toggle_magnifier(GtkCheckMenuItem *checkmenuitem,
   gpointer user_data)
 {
-   if (checkmenuitem->active)
+   if (gtk_check_menu_item_get_active (checkmenuitem))
        use_magnifier = TRUE;
    else
        use_magnifier = FALSE;
@@ -2638,7 +2637,7 @@ void _toggle_magnifier(GtkCheckMenuItem *checkmenuitem,
 void _toggle_festival(GtkCheckMenuItem *checkmenuitem,
   gpointer user_data)
 {
-   if (checkmenuitem->active)
+   if (gtk_check_menu_item_get_active (checkmenuitem))
        use_festival = TRUE;
    else
        use_festival = FALSE;
@@ -2647,7 +2646,7 @@ void _toggle_festival(GtkCheckMenuItem *checkmenuitem,
 void _toggle_festival_terse(GtkCheckMenuItem *checkmenuitem,
   gpointer user_data)
 {
-   if (checkmenuitem->active)
+   if (gtk_check_menu_item_get_active (checkmenuitem))
      {
         say_role = FALSE;
         say_accel = FALSE;
@@ -2662,7 +2661,7 @@ void _toggle_festival_terse(GtkCheckMenuItem *checkmenuitem,
 void _toggle_trackmouse(GtkCheckMenuItem *checkmenuitem,
   gpointer user_data)
 {
-   if (checkmenuitem->active)
+   if (gtk_check_menu_item_get_active (checkmenuitem))
      {
         mouse_watcher_focus_id =
           atk_add_global_event_listener(_mouse_watcher,
@@ -2686,7 +2685,7 @@ void _toggle_trackmouse(GtkCheckMenuItem *checkmenuitem,
 void _toggle_trackfocus(GtkCheckMenuItem *checkmenuitem,
   gpointer user_data)
 {
-   if (checkmenuitem->active)
+   if (gtk_check_menu_item_get_active (checkmenuitem))
      {
        track_focus = TRUE;
        focus_tracker_id = atk_add_focus_tracker (_print_accessible);
